from ..common import *


class TD_IPSO_3302_04(CoAPTestCase):
    """
testcase_id: TD_IPSO_3302_04
uri: http://openmobilealliance.org/iot/lightweight-m2m-lwm2m
configuration: LWM2M_CFG_01
objective: 
  - Setting the writable resources of object 3302 (Presence) Instance 0 using JSON data format (11543)
  - This test has to be run for the following resources
  - - Busy to Clear delay
  - - Clear to Busy delay
pre_conditions: 
  - Device is registred with the Server
  - The current values of the Server Object (ID:1) Instance 0, are saved on the Server
  - The Client supports the Configuration C.3 (A superset of C.1 where server objects implements {Default Minimum PEriod, Default Maximum Period, Disable Timeout} additional resources)  
  - - LWM2M Server Object (ID = 1) Instance 0 with mandatory resources and Short Server ID = 1
  - - LWM2M Security Object (ID = 0) Instance 0 with mandatory resources and Bootstrap Server = False
  - - LWM2M Device Object (ID = 3) Instance 0 with mandatory resources. 
  - The Client supports IPSO Presence object (ID:3302) with mandatory resources. 
sequence:
  - step_id: 'TD_IPSO_3302_04_step_01'
    type: stimuli
    node: lwm2m_server
    description:
      - 'LwM2M server sends a WRITE request (CoAP PUT) on Presence object instance'
      - - Type = 0 (CON)
        - Code = 3 (PUT)

  - step_id: 'TD_IPSO_3302_04_step_02'
    type: check
    description:
      - 'Sent PUT request contains'
      - - Type=0 and Code=3
        - Non-empty payload = A serialized representation of Presentation object instance
        - content-format=application/vnd.oma.lwm2m+json
        - URI-Path option= 3302/0

  - step_id: 'TD_IPSO_3302_04_step_03'
    type: check
    description:
      - 'LwM2M client sends response containing'
      - - Code = 2.04 (Changed)
        - content-format=application/vnd.oma.lwm2m+json
        - empty payload

  - step_id: 'TD_IPSO_3302_04_step_04'
    type: verify
    node: lwm2m_server
    description:
      - 'LwM2M server indicates successful operation'
      """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='put')]

    def run(self):

        self.match('server', CoAP(type='con', code='put', opt=self.uri('/3302/0')), 'fail')
        self.match('server', CoAP(pl=Not(b'')), 'fail')
        self.match('server', CoAP(opt=Opt(CoAPOptionContentFormat('11543'))), 'fail')
        
        self.next()

        self.match('client', CoAP(code=Any(65, 68)), 'fail')
        self.match('client', CoAP(pl=(b'')), 'fail')



#!/usr/bin/env python3

from ttproto.ts_coap.proto_specific import CoAPTestcase
from ttproto.ts_coap.proto_templates import *

class TD_COAP_CORE_14 (CoAPTestcase):
    """Identifier:
TD_COAP_CORE_14
Objective:
Perform GET transaction containing several URI-Query options (CON mode)
Configuration:
CoAP_CFG_BASIC
References:
[COAP] 5.4.5, 5.10.2,6.5

Pre-test
conditions:
•	Server offers a /query resource with resource content is not empty

Test Sequence:
Step
Type
Description

1
Stimulus
Client is requested to send a confirmable GET request with three Query parameters (e.g. ?first=1&second=2&third=3) to the server’s resource

2
Check
The request sent by the client contains:
•	Type = 0 (CON)
•	Code = 1 (GET)
•	Client-generated Message ID (➔ CMID)
•	Client-generated Token (➔ CTOK)
•	Uri-Path option "query"
and two options of Uri-Query, with values such as:
•	first=1
•	second=2


3
Check
Server sends response containing:
•	Code = 69 (2.05 content)
•	Message ID = CMID, Token = CTOK
•	Content-format option
•	Non-empty Payload

4
Verify
Client displays the response

"""
    def run (self):
        if self.urifilter:
            uri_query_opt = self.uri ("/query?first=1&second=2")
        else:
            uri_query_opt = Opt(CoAPOptionUriQuery(), superset=True)

        self.match_coap ("client", CoAP (code = "get",
                        type = "con",
                        opt = uri_query_opt))
        CMID = self.frame.coap["mid"]
        CTOK = self.frame.coap["tok"]

        opts = list (filter ((lambda o: isinstance (o, CoAPOptionUriQuery)), self.frame.coap["opt"]))

        if len (opts) < 2:
            self.setverdict ("inconclusive", "expect multiple UriQuery options")

        self.next_skip_ack()

        self.match_coap ("server", CoAP (	code = 2.05,
                            mid=CMID,
                            tok=CTOK,
                            pl = Not (b""),
                            opt = Opt (CoAPOptionContentFormat()),
                ))




#!/usr/bin/env python3

from ttproto.ts_coap.proto_specific import CoAPTestcase
from ttproto.ts_coap.proto_templates import *


class TD_COAP_CORE_10 (CoAPTestcase):
    """Identifier:
TD_COAP_CORE_10
Objective:
Perform GET transaction containing non-empty Token (CON mode)
Configuration:
CoAP_CFG_BASIC
References:
[COAP] clause 2.2 ,5.8.1, 5.10.1

Pre-test
conditions:
•	Server offers a /test resource with resource content is not empty that handles GET

Test Sequence:
Step
Type
Description

1
Stimulus
Client is requested to send a GET request to server’s resource including Token option

2
Check
The request sent by the client contains:
•	Type = 0 (CON)
•	Code = 1 (GET)
•	Option Type = Token
•	Client-generated Message ID (➔ CMID)
•	Client-generated Token (➔ CTOK)
•	Length of the token should be between 1 to 8 Bytes
•	Uri-Path option "test"


3
Check
Server sends response containing:
•	Code = 69 (2.05 content)
•	Message ID = CMID, Token = CTOK
•	Content-format option
•	Non-empty Payload

4
Verify
Client displays the response
"""
    def run (self):
        self.match_coap ("client", CoAP (	code = "get",
                            type = "con",
                            tok = Not (b""),
                            opt = self.uri ("/test")
        ))
        self.match_coap ("client", CoAP (tok = Length (bytes, (1, 8))
                ), "fail")
        CMID = self.frame.coap["mid"]
        CTOK = self.frame.coap["tok"]

        self.next()

        if self.match_coap ("server", CoAP (	code = 2.05,
                            pl  = Not (b"")
                )):
            self.match_coap ("server", CoAP (mid= CMID,tok = CTOK,opt= Opt(CoAPOptionContentFormat())), "fail")





#!/usr/bin/env python3
#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import io, sys, types
from xml.sax import saxutils

from	ttproto.core.typecheck	import *

__all__ = [
	'XMLGeneratorControl',
	'XMLGenerator',
	'XHTML10Generator',
]

class XMLGeneratorControl:
	def __init__(self, generator):
		self.__gen = generator

	def flush (self):
		self.__gen._XMLGenerator__file.flush()

	def write (self, txt):
		self.__gen._XMLGenerator__file.write (saxutils.escape (txt))

	def raw_write (self, txt):
		self.__gen._XMLGenerator__file.write (txt)

class XMLGenerator:
	@typecheck
	def __init__ (self, root:	str,
			output:		either (str, io.TextIOBase)	= sys.stdout,
			indented:	bool				= False,
			encoding:	str				= "UTF-8",
			dtd:		optional (tuple_of (str))	= None,
			attr:		dict_of (str, str)		= {}):

		if dtd:
			assert 2 <= len (dtd) <= 3
			assert dtd[0] in ("SYSTEM", "PUBLIC")

		self.__root = root
		self.__root_attr = attr

		if isinstance (output, str):
			self.__file = open (output, "w")
			self.__autoclose = True
		else:
			self.__file = output
			self.__autoclose = False


		self.__last = None
		self.__previous = None
		self.__parent = None
		self.__root_elem = None
		self.__level = 0

		if not indented:
			self.__write     = self.__file.write
			self.__write_txt = self.__file.write

		self.__write ('<?xml version="1.0" encoding="%s"?>' % encoding)

		self.__write ('<!DOCTYPE %s %s>' % (root,
				(('%s "%s" "%s"' if len (dtd) == 3 else '%s "%s"') % dtd)
			))


	def __enter__ (self):
		self.__root_elem = True	# to avoid assertion error in __getattr__
		self.__root_elem = getattr (self, self.__root)
		self.__root_elem (**self.__root_attr)
		self.__root_elem.__enter__()
		return self

	def __exit__ (self, a, b, c):
		self.__root_elem.__exit__(a, b, c)
		if self.__autoclose:
			self.__file.close()

	def __push (self, elem):
		# must be a child of elem.__parent
		assert elem._Elem__next == self.__parent
		assert self.__last == elem
		if repr(elem) == "<a>":
			raise

		self.__flush_previous()

		self.__last = None

		self.__parent = elem

		self.__status("doc push")

	def __pop (self, next_elem):

		self.__flush_last()
		self.__parent = next_elem

		self.__status("doc pop")

	def __replace (self, new_last):
		# must be a child of elem.__parent
		assert new_last._Elem__parent == self.__last

		self.__flush_previous()

		self.__last = new_last
		self.__status("doc replace")


	def __getattr__ (self, name):
		assert self.__root_elem		# must call __enter__ first

		self.__flush_previous()

		self.__previous = self.__last
		self.__last = XMLGenerator.__Elem (self, self.__parent, name)
		self.__status("doc attr")
		return self.__last

	def __status (self, txt):
#		print ("\t\t\t%-10s:	elem=%-10s last=%-10s previous=%-10s" % (txt, self.__parent, self.__last, self.__previous))
		pass

	def __flush_previous (self):
		if self.__previous:
			last, prev = self.__last, self.__previous
			self.__last, self.__previous = None, None
			prev._Elem__flush()
			self.__last = last

		self.__status("doc flush prev")

	def __flush_last (self, last = None):
		self.__flush_previous()
		if self.__last and self.__last is not last:
			self.__last._Elem__flush()
		self.__last = None
		self.__status("doc flush last")

	@typecheck
	def __call__ (self, txt):
		self.__write_txt (saxutils.escape (str (txt)))

	def __write_txt (self, txt):
		if txt[-1] == "\n":
			self.__write (txt[:-1])
		else:
			self.__write (txt)

	@typecheck
	def __write (self, txt: str):
		assert self.__level >= 0

		print (" " * self.__level, txt, sep="", file=self.__file)

	def __write_enter (self, txt):
		self.__write (txt)
		self.__level += 1

	def __write_exit (self, txt):
		self.__level -= 1
		self.__write (txt)

	class __Elem:
		def __init__ (self, generator, next_, name):
			self.__generator = generator
			self.__next = next_
			self.__name = name
			self.__flushed = False
			self.__attr = ""
			self.__parent = None

		def __call__ (__self, __txt = None, **__kw):
			if __kw:
				assert not __self.__attr

				__self.__attr = " " + " ".join ((("%s=\"%s\"" % (k,v)) for k,v in __kw.items()))

			if __txt is None:
				return __self
			else:
				assert __self.__flushed == False

				__self.__generator._XMLGenerator__flush_last (__self)
				try:
					__self.__open()
					__self.__generator._XMLGenerator__write (saxutils.escape (str (__txt)))
				finally:
#					print ("\t\t\tfinally call", sys.exc_info())
					__self.__close()

				return None

		def __enter__ (self):
			self.__generator._XMLGenerator__push (self)

			self.__open()

		def __open (self):
			assert not self.__flushed

			self.__generator._XMLGenerator__write_enter ("<%s%s>" % (self.__name, self.__attr))
			self.__flushed = True


		def __flush (self):
			assert not self.__flushed
			self.__generator._XMLGenerator__flush_last (self)
			self.__generator._XMLGenerator__write ("<%s%s/>" % (self.__name, self.__attr))
			self.__flushed = True
			self.__name = None

			self.__close()

		def __exit__ (self, a=None, b=None, c=None):
#			print ("\t\t\texit", self, a, b, c)

			if self.__name:
				self.__close()

			self.__generator._XMLGenerator__pop (self.__next)

		def __close (self):
			if self.__name:
				self.__generator._XMLGenerator__flush_last()
				self.__old_name = self.__name

				self.__generator._XMLGenerator__write_exit ("</%s>" % self.__name)
				self.__name = None

			if self.__parent:
				self.__parent.__close()
				self.__parent = None # FIXME: is this necessary ?

		def __getattr__ (self, name):
			child = type(self) (self.__generator, self.__next, name)
			child.__parent = self

			self.__generator._XMLGenerator__replace (child)
			self.__open()
			return child

		def __repr__ (self):
			return "<%s>" % (self.__name if self.__name else self.__old_name)


def XHTML10Generator (variant = "strict", **kw):
	assert variant in ("strict", "transitional", "frameset")
	assert "dtd" not in kw
	if "attr" in kw:
		assert xmlns not in kw["attr"]
	else:
		kw["attr"] = {}

	if "root" not in kw:
		kw["root"] = "html"

	kw["attr"]["xmlns"] = "http://www.w3.org/1999/xhtml"

	if variant == "strict":
		kw["dtd"] = ("PUBLIC", "-//W3C//DTD XHTML 1.0 Strict//EN", "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd")
	elif variant == "transitional":
		kw["dtd"] = ("PUBLIC", "-//W3C//DTD XHTML 1.0 Transitional//EN", "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd")
	elif variant == "frameset":
		kw["dtd"] = ("PUBLIC", "-//W3C//DTD XHTML 1.0 Frameset//EN", "http://www.w3.org/TR/xhtml1/DTD/xhtml1-frameset.dtd")
	else:
		raise # not reachable

	return XMLGenerator (**kw)


if __name__ == "__main__":

	with XHTML10Generator() as gen:

		with gen.head:
			gen.title("pouet")

		with gen.body:
			gen.a(href="http://www.irisa.fr/tipi").img(src="truc.jpg", alt="pouet")
			with gen.p.ul:
				gen.li("truc")
				gen.li("machin")
				gen.li("bidule")
			gen.hr

			with gen.p:
				gen("bonjour\n")
				gen("\n")
				gen("ça va ?\n")




import unittest
import requests
import base64
import hashlib

from collections import OrderedDict
from ttproto.core.control import Verdict


class StructureValidator(unittest.TestCase):

    # #################### Object structures #########################
    STRUCT_RESPONSE_OK = {
        '_type': str,
        'ok': bool,
        'content': list
    }
    STRUCT_RESPONSE_KO = {
        '_type': str,
        'ok': bool,
        'error': str
    }
    STRUCT_TC_BASIC = {
        '_type': str,
        'id': str,
        'objective': str
    }
    STRUCT_TC_IMPLEMENTATION = {
        '_type': str,
        'implementation': str
    }
    STRUCT_IMPLEMENTED_PROTOCOL = {
        '_type': str,
        'name': str,
        'description': str
    }
    STRUCT_TOKEN = {
        '_type': str,
        'value': str
    }
    STRUCT_FRAME = {
        '_type': str,
        'id': int,
        'timestamp': float,
        'error': (type(None), str),
        'protocol_stack': list
    }
    STRUCT_VERDICT = {
        '_type': str,
        'verdict': str,
        'description': str,
        'review_frames': list
    }
    VERDICT_VALUES = Verdict.values()

    # #################### Utilities functions #########################

    def check_correct_structure(self, el, structure):
        # Check that it's a non empty dict
        self.assertIsInstance(el, dict)
        self.assertGreater(len(el), 0)

        # Check its fields
        self.assertEqual(el.keys(), structure.keys())

        # Check the type of all its fields
        for field in structure:
            self.assertIsInstance(el[field], structure[field])

    def check_correct_response_header(self, response):
        # Check the object type
        self.assertIsInstance(response, requests.models.Response)

        # Check the response code
        self.assertEqual(response.status_code, 200)

        # Check the headers
        self.assertEqual(
            response.headers['content-type'],
            'application/json;charset=utf-8'
        )

    def check_request_not_found_header(self, response):
        # Check the object type
        self.assertIsInstance(response, requests.models.Response)

        # Check the response code
        self.assertEqual(response.status_code, 404)

    def check_correct_response(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_RESPONSE_OK)

        # Check its values
        self.assertEqual(el['_type'], 'response')
        self.assertTrue(el['ok'])
        self.assertGreater(len(el['content']), 0)

    def check_error_response(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_RESPONSE_KO)

        # Check its values
        self.assertEqual(el['_type'], 'response')
        self.assertFalse(el['ok'])

    def check_tc_basic(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_TC_BASIC)

        # Check its type
        self.assertEqual(el['_type'], 'tc_basic')

    def check_tc_implementation(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_TC_IMPLEMENTATION)

        # Check its type
        self.assertEqual(el['_type'], 'tc_implementation')

    def check_implemented_protocol(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_IMPLEMENTED_PROTOCOL)

        # Check its type
        self.assertEqual(el['_type'], 'implemented_protocol')

    def check_token(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_TOKEN)

        # Check its type
        self.assertEqual(el['_type'], 'token')

        # Check that the value is a correct hash
        # Add '=' only for checking
        decoded = base64.urlsafe_b64decode(el['value'] + '=')
        self.assertEqual(len(decoded), hashlib.sha1().digest_size)

    def check_protocol(self, el):
        # Check that it's a non empty dict
        self.assertIsInstance(el, dict)
        self.assertGreater(len(el), 0)

        # Check its minimum fields
        self.assertIn('_type', el)
        self.assertIn('_protocol', el)
        self.assertEqual(el['_type'], 'protocol')

        # Check that it has more values than that
        self.assertGreater(len(el), 2)

    def check_frame(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_FRAME)

        # Check its type
        self.assertEqual(el['_type'], 'frame')

        # Check that we have a protocol stack
        self.assertGreater(len(el['protocol_stack']), 0)
        for prot in el['protocol_stack']:
            self.check_protocol(prot)

    def check_is_int_real(self, el):
        # Check its type
        self.assertEqual(type(el), int)

        # Check its value
        self.assertGreater(el, 0)

    def check_verdict(self, el):
        # Check the structure
        self.check_correct_structure(el, self.STRUCT_VERDICT)

        # Check its type
        self.assertEqual(el['_type'], 'verdict')

        # Check that the verdict given is contained in the accepted values
        self.assertIn(el['verdict'], self.VERDICT_VALUES)

        # Check that we have some review frames
        self.assertGreaterEqual(len(el['review_frames']), 0)
        for frame in el['review_frames']:
            self.check_is_int_real(frame)

    def check_tc_from_analyzer(self, el):

        # Check that we receive a list of tuple
        self.assertEqual(type(el), list)

        for tc in el:
            self.assertEqual(type(tc), tuple)
            self.assertEqual(len(tc), 4)

            # For each element of this, it should be a string
            for tuple_el in tc:
                self.assertEqual(type(tuple_el), str)

    def check_tc_from_webserver(self, el):

        # Webserver function return an ordered dict
        self.assertEqual(type(el), OrderedDict)

        # For each couple key / value
        for key in el:

            # Get the value
            value = el[key]

            # Check that the key is a name and value a dict
            self.assertEqual(type(key), str)
            self.assertEqual(type(value), OrderedDict)

            # Check the fields of value
            self.assertEqual(len(value), 2)
            self.assertIn('tc_basic', value)
            self.assertIn('tc_implementation', value)

            # Check the two fields then
            self.check_tc_basic(value['tc_basic'])
            self.check_tc_implementation(value['tc_implementation'])



#!/usr/bin/env python3
#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

EMULATION_MODE	= True

# ipv6calc --in prefix+mac --action prefixmac2ipv6 fe80::/10 00:00:00:00:00:42 --out ipv6addr
# ipv6calc --in mac --action geneui64 00:00:00:00:00:42 --out eui64

PRFX1 = "2001:0db8:1:1::"

PRFX2 = "fdea:1ffc:f851::" # Generated with http://www.sixxs.net/tools/grh/ula/

#NUT_addr	= "::"
#NUT_lladdr	= "fe80::74d7:4148:1000:29"
#NUT_hwaddr	= "76:d7:41:48:10:00:00:29"
#NUT_eui64	= "76:d7:41:48:10:00:00:29"

#NUT_addr	= "::"
#NUT_lladdr	= "fe80::74d7:4148:1000:2F"
#NUT_hwaddr	= "76:d7:41:48:10:00:00:2F"
#NUT_eui64	= "76:d7:41:48:10:00:00:2F"

NUT_addr	= "2001:db8:1:1::102:304"
NUT_lladdr	= "fe80::102:304"
NUT_hwaddr	= "0200000001020304"
NUT_eui64	= "0200000001020304"

#LBR1_lladdr	= "fe80::200:0:0:42"
#LBR1_hwaddr	= "0000000000000042"
#LBR1_eui64	= "0000000000000042"

LBR1_addr	= "2001:db8:1:1::ff:fe00:aa76"
LBR1_lladdr	= "fe80::ff:fe00:aa76"
LBR1_hwaddr	= "aa76"
LBR1_eui64	= "0200000102030405"

LBR2_addr	= "2001:db8:1:1::ff:fe00:aa77"
LBR2_lladdr	= "fe80::ff:fe00:aa77"
LBR2_hwaddr	= "aa77"
LBR2_eui64	= "0200000102030405"

LR1_addr	= "2001:db8:1:1::ff:fe00:aa78"
LR1_lladdr	= "fe80::ff:fe00:aa78"
LR1_hwaddr	= "aa78"
LR1_eui64	= "0200000102030405"

LR2_addr	= "2001:db8:1:1::ff:fe00:aa79"
LR2_lladdr	= "fe80::ff:fe00:aa79"
LR2_hwaddr	= "aa77"
LR2_eui64	= "0200000102030405"

HOST1_addr	= "2001:db8:1:1::ff:fe00:aa80"
HOST1_lladdr	= "fe80::ff:fe00:aa80"
HOST1_hwaddr	= "aa80"
HOST1_eui64	= "0200000102030405"

HOST2_addr	= "2001:db8:1:1::ff:fe00:aa81"
HOST2_lladdr	= "fe80::ff:fe00:aa81"
HOST2_hwaddr	= "aa81"
HOST2_eui64	= "0200000102030405"

TIMER1		= 30 # time to wait when we except a packet
TIMER2		= 30 # time to wait when we don't except a packet

SLEEP1		= 1

PANID		= 0x7600



from ..common import *
from ...core.templates import Range

class TD_COAP_BLOCK_06(CoAPTestCase):
    """
---
TD_COAP_BLOCK_06:
    obj: Handle GET blockwise transfer for large resource (early negotiation, 16 byte block size)
    cfg: CoAP_CFG_BASIC
    ref: '[BLOCK] 2.2-2.4'

    pre:
        - Client supports Block2 transfers
        - Server supports Block2 transfers
        - Server offers a large resource /large
        - Client does not know /large requires block transfer

    seq:
    -   s: "Client is requested to retrieve resource /large"

    -   c:
        - 'Client sends a GET request not containing Block2 option indicating:'
        -   - NUM = 0
            - M = 0
            - "SZX (➔DES_SZX) is the desired block size"

    -   c:
        - 'Server sends 2.05 (Content) response with a Block2 option
        indicating:'
        -   - NUM = 0
            - M = 1
            - "SZX = (➔ACT_SZX) is less than or equal to DES_SZX."
            - "Payload size is 2SZX+4 bytes"

    -   c:
        - 'Client send GET requests for further blocks indicating:'
        -   - NUM = i where “i” is the block number of the current block
            - M = 0
            - SZX is ACT_SZX

    -   c:
        - 'Server sends 2.05 (Content) response with a Block2 option
        indicating:'
        -   - NUM = i where “i” is the block number used at step 4
            - M = 1
            - SZX is the ACT_SZX
            - Payload size is 2SZX+4 bytes

    -   c:
        - 'Client send GET request for the last block indicating:'
        - NUM = n where “n” is the last block number;
        - M = 0;
        - SZX is the ACT_SZX

    -   c:
        - 'Server sends 2.05 (Content) response with a Block2 option
        indicating:'
        - NUM = n where “n” is the block number used at step 8
        - M = 0
        - SZX is the ACT_SZX
        - Payload size is lesser or equal to 2SZX+4

    -   v: Client displays the received information

    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='con', code='get', opt=Opt(CoAPOptionUriPath("large"),CoAPOptionBlock2())),
        ]

    def run (self):
        # Step 2 - initial request from the client
        self.match("client", CoAP(code = "get",
                opt = self.uri("/large", Opt(CoAPOptionBlock2(num=0,m=0)))))
        client_szx  = self.coap["opt"][CoAPOptionBlock2]["szx"]
        client_num  = self.coap["opt"][CoAPOptionBlock2]["num"]
        self.next_skip_ack()


        # Step 3 - response from the server w/ block size negociation
        self.match("server", CoAP (code = 2.05,
            opt = Opt(CoAPOptionBlock2(num=0,szx=Range(int, 0, client_szx)))
                    ), "fail")
        server_szx  = self.coap["opt"][CoAPOptionBlock2]["szx"]
        server_size = 2**(server_szx+4)

        # Step 3 - expect more blocks
        self.match("server", CoAP(opt = Opt(CoAPOptionBlock2(m=1)),
                                    pl = Length(bytes, server_size)
        ))

        self.next_skip_ack()

        verdict_if_none = "inconclusive"
        client_num += 1
        # Step 4 - request next block from client
        while self.match("client", CoAP(code = "get",
                    opt = Opt(CoAPOptionBlock2(num=client_num, szx=server_szx)))
                    , verdict_if_none):

            self.match("client", CoAP (opt = Opt(CoAPOptionBlock2(m=0))),
                        "fail")

            self.next_skip_ack()
            # Step 5 - send next block from server
            if not self.match("server", CoAP(code = 2.05,
                            opt = Opt(CoAPOptionBlock2(num=client_num, szx=server_szx))
                    )):
                break
            if not self.match("server", CoAP(opt = Opt(CoAPOptionBlock2(m=1)),
                                                pl = Length(bytes, server_size)
                                ), None):
                # Step 7 - no more blocks
                self.match("server", CoAP (opt = Opt(CoAPOptionBlock2(m=0)),
                                            pl = Length(bytes, (1, server_size))
                            ), "fail")
                verdict_if_none = None
                # end of testcase
                self.next_skip_ack(optional = True)
                break
            self.next_skip_ack()

            # The expected block number increase for each block.
            client_num += 1



# -*- coding:utf-8 -*-
import os
import errno
import logging

__version__ = '0.1.11'

PACKAGE_DIR = os.path.dirname(os.path.realpath(__file__))
# Directories
DATADIR = os.path.join(PACKAGE_DIR, "data")
TMPDIR = os.path.join(PACKAGE_DIR, "tmp")
LOGDIR = os.path.join(PACKAGE_DIR, "log")

LOG_LEVEL = logging.INFO
LOG_FORMAT = '%(levelname)s %(name)s [%(threadName)s] %(message)s'
logging.basicConfig(level=LOG_LEVEL, format=LOG_FORMAT)

# lower versbosity of pika's logs
logging.getLogger('pika').setLevel(logging.WARNING)

logging.info("ttproto base dir : %s " % PACKAGE_DIR)

for d in TMPDIR, DATADIR, LOGDIR:
    logging.info("Creating %s directory." % d)
    try:
        os.makedirs(d)
    except OSError as e:
        if e.errno != errno.EEXIST:
            raise



#!/usr/bin/env python3

from ..common import *

class TD_COAP_ANALYSIS_1 (CoapPrivacyTestCase):
    """
    Identifier:
        TD_COAP_ANALYSIS_1
    Objective:
        Analyse all GET messages and look for keywords referring to potential private sensitive information.

    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]

        .. warning::
            For the moment, we didn't manage to generate packets with the
            wanted size so we just don't take them into account
        """
        return []



    def run(self):
        while self.next():
            self.match ("client", CoAP (type="con", code="get"))




from ..common import *


class TD_COAP_CORE_04(CoAPTestCase):
    """
---
TD_COAP_CORE_04:
    cfg: CoAP_CFG_BASIC
    obj: Perform POST transaction (CON mode)
    pre: Server accepts POST request on /test
    ref: '[COAP] 5.8.2, 1.2, 2.1, 2.2, 3.1'
    seq:
    -   s:
        - 'Client is requested to send a POST request with:'
        -   - Type = 0 (CON)
            - Code = 2(POST)
            - Content-format option
            - Empty or non-empty Payload
    -   c:
        - 'The request sent by the client contains:'
        -   - Type=0 and Code=2
            - "Client-generated Message ID (\u2794 CMID)"
            - "Client-generated Token (\u2794 CTOK)"
            - Uri-Path option "test"
    -   v: Server displays received information
    -   c:
        - 'Server sends response containing:'
        -   - Code = 2.01 (Created) or 2.04 (Changed)
            - Message ID = CMID, Token = CTOK
            - Content-format option if payload non-empty
            - Zero or more Location-path options
            - Empty or non-empty Payload
    -   v: Client displays the received response
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='con', code='post', opt=Opt(CoAPOptionUriPath("test")))
        ]

    def run(self):
        self.match("client", CoAP(type="con", code="post", opt=self.uri("/test")))
        self.match("client", CoAP(opt=Opt(CoAPOptionContentFormat()), ), "fail")
        CMID = self.coap["mid"]
        CTOK = self.coap["tok"]

        self.next()

        self.match("server", CoAP(code=Any(65, 68), mid=CMID, tok=CTOK, ))

        if self.match("server", CoAP(pl=Not(b"")), None):
            self.match("server", CoAP(opt=Opt(CoAPOptionContentFormat()), ), "fail")



from ..common import *
from ...core.templates import Length, Range


class TD_COAP_BLOCK_02(CoAPTestCase):
    """
---
TD_COAP_BLOCK_02:
    obj: Handle GET blockwise transfer for large resource (late negotiation)
    cfg: CoAP_CFG_BASIC
    ref: '[BLOCK] 2.2-2.4'

    pre:
        - Client supports Block2 transfers
        - Server supports Block2 transfers
        - Server offers a large resource /large
        - Client does not know /large requires block transfer

    seq:
    -   s: "Client is requested to retrieve resource /large"

    -   c: Client sends a GET request not containing Block2 option

    -   c:
        - 'Server sends 2.05(Content) response with a Block2 option
        indicating:'
        -   - NUM = 0;
            - M = 1;
            - SZX = the proposed block size.
            - Payload size is 2SZX+4 bytes.

    -   c:
        - 'Client switches to blockwise transfer mode and sends a GET
        request with a Block2 option indicating:'
        -   - "NUM is the next block number(should be equal to
            2SZX_in_step_4 – SZX_in_step_3)"
            -    M = 0
            -    SZX is less or equals to SZX at step 3


    -   c:
        - 'Server sends 2.05(Content) response with a Block2 option
        indicating:'
        -   - NUM = k where “k” is the block number used at step 4
            - M = 1
            - SZX is the SZX at step 4
            - Payload size is 2SZX+4 bytes

    -   c:
        - 'Client sends GET request for further blocks indicating:'
        -   - 'NUM = i where “i” is the block number of the current block'
            - M = 0
            - SZX is the SZX at step 4

    -   c:
        - 'Server sends 2.05(Content) response with a Block2 option indicating:'
        -   - NUM = i where “i” is the block number used at step 6
            - M = 1
            - SZX is the SZX at step 4
            - Payload size is 2SZX+4 bytes

    -   c:
        -   'Client send GET request for the last block indicating:'
        -   - NUM = n where “n” is the last block number
            - M = 0
            - SZX is the SZX at step 4

    -   c:
        -   'Server sends 2.05(Content) response with a Block2 option
            indicating:'
        -   - NUM = n where “n” is the block number used at step 8
            - M = 0
            - SZX is the SZX at step 4
            - Payload size is lesser or equal to 2SZX+4

    -   v: Client displays the received information

    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='con', code='get', opt=Opt(CoAPOptionUriPath("large"), Not(CoAPOptionBlock())))
        ]

    def run(self):
        # Step 2 - initial request from the client
        self.match("client", CoAP(code = "get",
                         opt = self.uri("/large", NoOpt(CoAPOptionBlock2()))))

        self.next_skip_ack()

        # Step 3 - response from the server w/ block size negociation
        self.match("server", CoAP(code = 2.05,
                         opt = Opt(CoAPOptionBlock2(num=0))
                ), "fail")
        server_szx  = self.coap["opt"][CoAPOptionBlock2]["szx"]
        server_size = 2**(server_szx+4)

        # Step 3 - expect more blocks
        self.match("server", CoAP(opt = Opt(CoAPOptionBlock2(m=1)),
                         pl = Length(bytes, server_size)
                ))

        self.next_skip_ack()

        # Step 4 - second request from the client.
        self.match("client", CoAP(code = "get",
                         opt = Opt(CoAPOptionBlock2(szx=Range(int, 0, server_szx)))))

        client_szx  = self.coap["opt"][CoAPOptionBlock2]["szx"]
        client_size = 2**(client_szx+4)

        client_num  = self.coap["opt"][CoAPOptionBlock2]["num"]

        max_next_num = 2 **(server_szx - client_szx)
        if client_num > max_next_num:
            self.set_verdict("inconclusive", "discontinuity in client requests(next requested block should be between 1 and %d)" % max_next_num)

        for i in itertools.count(client_num):

            # Step 6,  Step 8 (last iteration)
            # request next block from client
            if self.match("client", CoAP(code = "get",
                             opt = Opt(CoAPOptionBlock2(num=i, szx=client_szx)))):

                self.match("client", CoAP(opt = Opt(CoAPOptionBlock2(m=0))), "fail")

            self.next_skip_ack()

            # Step 5 (first iteration), Step 7
            # send next block from server
            if not self.match("server", CoAP(code = 2.05,
                             opt = Opt(CoAPOptionBlock2(num=i, szx=client_szx))
                    )):
                break

            if not self.match("server", CoAP(opt = Opt(CoAPOptionBlock2(m=1)),
                                pl = Length(bytes, client_size)
                        ), None):
                # Step 9 - no more blocks
                self.match("server", CoAP(opt = Opt(CoAPOptionBlock2(m=0)),
                                pl = Length(bytes,(1, client_size))
                        ), "fail")

                # end of testcase
                self.next_skip_ack(optional = True)
                break

            self.next_skip_ack()



import os
import unittest
from ttproto.tat_coap.common import CoAPTestCase
from ttproto.core.dissector import Capture
from ttproto.tat_coap.testcases.td_coap_core_04 import TD_COAP_CORE_04
from ttproto.tat_coap.testcases.td_coap_core_09 import TD_COAP_CORE_09
from ttproto.tat_coap.testcases.td_coap_obs_07 import TD_COAP_OBS_07
from ttproto.core.analyzer import Conversation
# from ttproto.core.lib.inet.coap import *
from ttproto.core.lib.inet.udp import UDP
from ttproto.tat_coap.templates import CoAP, CoAPOptionUriPath
from ttproto.core.typecheck import typecheck


class TestCommon(unittest.TestCase):

    def __get_capture(self,
                      pcap_file,
                      dir_from_test_dumps="coap_observe"):
        pcap_dir_path = os.path.dirname(os.path.abspath(__file__))
        pcap_dir_path = os.path.join(pcap_dir_path,
                                     os.path.join('../test_dumps/'
                                                  , dir_from_test_dumps)
                                     )
        return Capture(os.path.join(pcap_dir_path, pcap_file))

    def __get_convs(self,
                    pcap_file:str,
                    dir_from_test_dumps='preprocess/coap'):
        capture = self.__get_capture(pcap_file, dir_from_test_dumps)
        convs, ignored = CoAPTestCase.extract_all_coap_conversations(capture)
        return convs, ignored

    def test_conv_with_no_token(self):
        convs, ignored = self.__get_convs('Single_conversation_empty_token.pcap')
        assert len(convs) == 1
        self.__check_single_conv_no_token(convs[0])

    def test_convs_with_token(self):
        convs, ignored = self.__get_convs('Two_conversations_two_tokens.pcap')
        assert len(convs) == 2

        # For each conversation, we want to check that all packets have the same
        # token. However, we do ignore token in case of ACK packet as those do
        # not have token value.
        # We still check that the CMID of the ACK is a CMID of a frame with the
        # current conversation token to verify that this ACK really belong to it.
        for conv in convs:
            self.__check_single_conv_with_token(conv)

    def test_convs_mixed(self):
        convs, ignored = self.__get_convs('Two_conversations_one_token.pcap')
        assert len(convs) == 2
        self.__check_single_conv_no_token(convs[0])
        self.__check_single_conv_with_token(convs[1])

    def test_conv_with_RST(self):
        capture = self.__get_capture('TD_COAP_OBS_06_PASS_One_Con_before_RST.pcap')
        convs, ignored = CoAPTestCase.extract_all_coap_conversations(capture)

        # We want only one conversation here : The last packet, containing the
        # RST, belong to the same conversation of all previous packets in the
        # capture TD_COAP_OBS_06_PASS_One_Con_before_RST.pcap.
        # It is because the RST has the same an MID as the last packet
        # sent by the server, hence the RST belong to the same conversation
        # described by the token of this last packet
        assert len(convs) == 1

    def __check_single_conv_with_token(self, conv):
        first_frame = conv[0]
        current_conversation_CTOK = first_frame[CoAP]["tok"]
        CMID = first_frame[CoAP]['mid']

        # The set of encountered CMID in this conversation.
        # When we see an ACK, the CMID of this ACK must be already present
        # in this set. If its not the case, it means that this ACK should
        # not belong in this conversation.
        CMIDs = set()
        # CMIDs.add(CMID)

        for frame in conv:
            current_frame_CTOK = frame[CoAP]["tok"]

            if current_frame_CTOK == b'':
                # ACK do not have a token and are solely based on message ID.
                # For this TC, empty token are acceptable only for ACK.
                assert frame[CoAP]['type'] == 2
                assert frame[CoAP]['mid'] in CMIDs
            else:
                assert current_frame_CTOK == current_conversation_CTOK
                CMID = frame[CoAP]['mid']
                # assert CMID not in CMIDs
                CMIDs.add(CMID)

    def __check_single_conv_no_token(self, conv):
        # The set of encountered CMID in this conversation.
        # When we see an ACK, the CMID of this ACK must be already present
        # in this set. If its not the case, it means that this ACK should
        # not belong in this conversation.
        CMIDs = set()

        for frame in conv:
            current_frame_CTOK = frame[CoAP]["tok"]
            assert current_frame_CTOK == b''
            if frame[CoAP]['type'] == 2:
                assert frame[CoAP]['mid'] in CMIDs
            else:
                CMID = frame[CoAP]['mid']
                # assert CMID not in CMIDs
                CMIDs.add(CMID)

    def test_conv_correlation_into_single_conv(self):
        capture = self.__get_capture('TD_COAP_OBS_07_PASS_With_202_Deleted_delayed.pcap')
        convs, ignored = CoAPTestCase.extract_all_coap_conversations(capture)
        # We await 2 conversations : One for the observing client on /obs,
        # one for another client doing a DELETE request on /obs.
        assert len(convs) == 2
        conv_obs_client = convs[0]
        assert len(convs[0]) == 8  # All the frame of the observing client.
        assert conv_obs_client[0][CoAP]["opt"][CoAPOptionUriPath]['val'] == 'obs'
        assert conv_obs_client[6][CoAP]['code'] == 132

        # The conversation of the client doing a DELETE req has only 2 frames
        # in the pcap :
        assert len(convs[1]) == 2
        conv_del_client = convs[1]
        # The delete request on /obs...
        assert conv_del_client[0][CoAP]['code'] == 4
        assert conv_del_client[0][CoAP]["opt"][CoAPOptionUriPath]['val'] == 'obs'
        # ...and the 2.02 DELETED confirmation from the server.
        assert conv_del_client[1][CoAP]['code'] == 66

        correlated_conv = CoAPTestCase.correlate(convs,
                                                 TD_COAP_OBS_07.get_stimulis()
                                                 )[0]
        assert len(correlated_conv) == 10
        for i in range(0, 5):
            # Before the stimulis of the second client (doing delete) happen,
            # The frames of the correlated conv are identical with the frames of
            # the conversation of the first client (doing get).
            assert correlated_conv[i][CoAP] == conv_obs_client[i][CoAP]

        # The second stimulis of the second client start here.
        # We check we managed properly some reordering here :
        # We want to have the confirmation "2.02 DELETED" sent to the second
        # client right after his DELETE request and before the "4.04 Not Found"
        # sent to the first client. Even if it's not  necessary the case in the
        # original pcap.

        # We also check that the stimulus frame of the second client is
        # still the seventh, as we're supposed to add the frames of the second
        # conversation into the correlated one as soon as we encounter it's
        # first stimulis.

        assert correlated_conv[6][CoAP]['code'] == 4 # Frame 7 - conversation 2
        assert correlated_conv[7][CoAP]['code'] == 66 # Frame 8 - conversation 2
        assert correlated_conv[7][CoAP]['type'] == 2

        assert correlated_conv[8][CoAP]['code'] == 132 # From conversation 1
        assert correlated_conv[9][CoAP]['mid'] == correlated_conv[8][CoAP]['mid']

    def test_conv_correlation_with_unrealated_conv_in_pcap(self):
        capture = self.__get_capture('OBS_07_with_other_unrelated_convs.pcap',
                                     dir_from_test_dumps='preprocess/coap')
        convs, ignored = CoAPTestCase.extract_all_coap_conversations(capture)

        # We await 2 conversations : One for the observing client on /obs, one for
        # another client doing a DELETE request on /obs, and 3 others unrealated
        # conversations (GET and POST on /test).
        assert len(convs) == 5

        all_correlated_convs = CoAPTestCase.correlate(convs,
                                                      TD_COAP_OBS_07.get_stimulis())
        # There is only that can realisticly match one instance of OBS_07 in the
        # pcap.
        assert len(all_correlated_convs) == 1
        # Those 8 frames are related to the TC.
        correlated_conv = all_correlated_convs[0]
        assert len(correlated_conv) == 8
        assert correlated_conv[0][CoAP]["opt"][CoAPOptionUriPath]['val'] == 'obs'

        # The delete request on /obs. It now the 5th frame and not the 9th like
        # in the original capture as 4 frames related to some core TC has been
        # filtered.
        assert correlated_conv[4][CoAP]['code'] == 4
        assert correlated_conv[4][CoAP]["opt"][CoAPOptionUriPath]['val'] == 'obs'
        # ...and the 2.02 DELETED confirmation from the server.
        # Because of reordering, it's right after the deletion request.
        assert correlated_conv[5][CoAP]['code'] == 66

        # 4.04 not found.
        assert correlated_conv[6][CoAP]['code'] == 132
        # Ack from client.
        assert correlated_conv[7][CoAP]['mid'] == correlated_conv[6][CoAP]['mid']

    def test_severals_convs_for_tc_in_single_pcap(self):
        capture = self.__get_capture('Two_tc_two_times_each_with_overlap.pcap',
                                     dir_from_test_dumps='preprocess/coap')
        conversations, ignored = CoAPTestCase.extract_all_coap_conversations(capture)
        assert len(conversations) == 4

        # A normal conversation of CORE 09 (separate response) is one get, one ack
        # from server, one notification (2.05 content) from server later and then
        # one ack from client for this notification.
        # Hence 2 conversation and 8 frames are expected (4 per conversation)
        # as the pcap contains two occurences of COAP_CORE_09 scenario.
        conversations_tc_09 = CoAPTestCase.correlate(
                                                conversations,
                                                TD_COAP_CORE_09.get_stimulis()
                                                    )
        # The pcap contains two correlated conversations related to CORE_09.
        assert len(conversations_tc_09) == 2

        assert len(conversations_tc_09[0]) == 4,\
            "expected 4 frames , but got %s " % len(conversations_tc_09[0])
        self.__check_conv_is_tc_09(conversations_tc_09[0])
        assert len(conversations_tc_09[1]) == 4,\
            "expected 4 frames , but got %s " % len(conversations_tc_09[1])
        self.__check_conv_is_tc_09(conversations_tc_09[1])

        # A normal conversation of CORE 04 is a simple POST with a 2.01 created
        # or a 2.04 changed response from the server. We have those two packets
        # for one execution of this case.
        # We did execute two times the stimulis in the given pcap, hence
        # 2 conversations and 4 frames are expected (2 per conversation).
        conversations_tc_04 = CoAPTestCase.correlate(
                                                conversations,
                                                TD_COAP_CORE_04.get_stimulis()
                                                    )
        assert len(conversations_tc_04) == 2,\
            "expected 2 conversations , but got %s " % len(conversations_tc_04)

        self.__check_conv_is_tc_04(conversations_tc_04[0])
        assert len(conversations_tc_04[1]) == 2,\
            "expected 2 frames , but got %s " % len(conversations_tc_04[0])

        self.__check_conv_is_tc_04(conversations_tc_04[1])
        assert len(conversations_tc_04[1]) == 2,\
            "expected 2 frames , but got %s " % len(conversations_tc_04[1])

    @classmethod
    @typecheck
    def __check_conv_is_tc_09(self, conv: Conversation):
        token = conv[0][CoAP]['tok']
        assert conv[0][CoAP]['code'] == 1
        assert conv[0][CoAP]["opt"][CoAPOptionUriPath]['val'] == 'separate'
        # print(conv[0][CoAP])
        # assert conv[0][CoAP].match(CoAP(type='con'))
        assert conv[1][CoAP]['type'] == 2
        assert conv[1][CoAP]['code'] == 0
        assert conv[1][UDP]['sport'] == 5683
        assert conv[2][CoAP]['code'] == 69
        assert conv[2][CoAP]['tok'] == token
        assert conv[2][UDP]['sport'] == 5683

        assert conv[3][CoAP]['type'] == 2
        assert conv[3][CoAP]['code'] == 0
        assert conv[3][UDP]['dport'] == 5683

    @classmethod
    @typecheck
    def __check_conv_is_tc_04(self, conv: Conversation):
        token = conv[0][CoAP]['tok']
        assert conv[0][CoAP]['code'] == 2
        assert conv[0][CoAP]["opt"][CoAPOptionUriPath]['val'] == 'test'

        assert conv[1][UDP]['sport'] == 5683
        assert conv[1][CoAP]['tok'] == token
        assert conv[1][CoAP]['type'] == 2
        assert conv[1][CoAP]['code'] == 65 # 2.01 Created
        assert conv[1][CoAP]["opt"][0]['val'] == 'location1'
        assert conv[1][CoAP]["opt"][1]['val'] == 'location2'
        assert conv[1][CoAP]["opt"][2]['val'] == 'location3'

    @classmethod
    @typecheck
    def __pprint_coap_conversation(cls, conv:Conversation):
        cls.__pprint_conversation(conv, 5683)

    @classmethod
    @typecheck
    def __pprint_conversation(cls, conv:Conversation, server_port_number:int):
        """
        print the given conversation with more detail about the presence of
        several differents sub-conversations contained inside (e.g several clients)
        It also show clearer the source and destination of Frame as "Client" and
        "Server" instead of IP addresses.
        """
        # TODO Probably relevant to move this method elsewhere. Maybe as class
        # method of Conversation
        conv_src_ports_to_conv_number = dict()
        print('\n')
        last_conv_num = -1
        for frame in conv:
            frameStr = str(frame)
            if frame[UDP]['sport'] != server_port_number:
                client_port = frame[UDP]['sport']
                server_ip = frameStr[frameStr.index('[')+1:frameStr.index('->')-1]
                client_ip = frameStr[frameStr.index('->')+3:frameStr.index(']')]
            else:
                client_port = frame[UDP]['dport']
                client_ip = frameStr[frameStr.index('[')+1:frameStr.index('->')-1]
                server_ip = frameStr[frameStr.index('->')+3:frameStr.index(']')]

            conv_number = conv_src_ports_to_conv_number.get(client_port)
            new_conversation = False
            if conv_number is None:
                conv_number = len(conv_src_ports_to_conv_number.keys()) + 1
                conv_src_ports_to_conv_number[client_port] = conv_number
                new_conversation = True

            if conv_number != last_conv_num:
                if last_conv_num != -1:
                    print("============== Next packet is not in conversation " + str(last_conv_num) + " ==============" +'\n')
                if not new_conversation:
                    print("============== In conversation " + str(conv_number)+ " once again ==============")
                else:
                    print("============== Begin  of conversation " + str(conv_number)+ " ==============")

            if (frame[UDP]['sport'] == server_port_number):
                client_port = frame[UDP]['dport']
                conv_number = conv_src_ports_to_conv_number[client_port]
                frameStr = frameStr.replace(server_ip + " ->", "Server ->")
                frameStr = frameStr.replace("-> " + client_ip, "-> Client " + str(conv_number))
            else:
                client_port = frame[UDP]['sport']
                conv_number = conv_src_ports_to_conv_number[client_port]
                frameStr = frameStr.replace(client_ip + " ->", "Client "  + str(conv_number) + " ->")
                frameStr = frameStr.replace("-> " + server_ip, "-> Server")
            print(frameStr + '\n')
            last_conv_num = conv_number

        print("============== End of conversation " + str(conv_number) + " ==============" +'\n')



#!/usr/bin/env python3
#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

from .common import *

#FIXME: remove this ... already in common.py but not working ..
from ttproto.core.list			import *
from ttproto.core.templates 		import *

def superop ( opt ):
	return Superset (ICMPv6OptionList , opt)

################################################################################
# CommonTestSetup_1_1
################################################################################

def CommonTestSetup_1_1( lbr , nut , prfx , timer1, link, routerlt = 60 , prefixlt = 60):
	#pass

	log ("Entering common test setup 1.1")
	t = Timer()

	unexpected_packets.activate( link )
	timeout.activate( t )

	ra = pack (
		IPv6( src = lbr.lladdr , dst = nut.lladdr ) ,
		ICMPv6RAdv( rlt = routerlt, opt = [
			ICMPv6SLL( hw=lbr.hwaddr ) ,
			SixPI( pf=prfx , vlt = prefixlt )
			]
		)
	)

	ereq = pack(
		IPv6 (src = lbr.addr, dst = nut.addr ),
		ICMPv6EchoRequest(),
		"6lowpan-nd-common-test-setup-1.1"
	)
	t.start(timer1)
	
	link.receive (
		IPv6( src = nut.lladdr , dst = IPV6_ALL_ROUTERS ) /
		ICMPv6RSol( )
	)

	link.send(ra)
	t.start(timer1)

	with alt:	#FIXME: removable
		@link.receive (
			IPv6( src = nut.addr , dst = lbr.lladdr ) /
			ICMPv6NSol( opt = superop ( [
				ICMPv6SLL( hw=nut.hwaddr ) ,
				SixARO()
				] )
			)
		)
		def _(value):
			na = pack (
				IPv6( src = lbr.lladdr , dst = nut.lladdr ) ,
				ICMPv6NAdv( opt = [
					SixARO( lt  = value[IPv6][ICMPv6]["opt"][SixARO]["lt"] , st = 0,
						eui = value[IPv6][ICMPv6]["opt"][SixARO]["eui"])
					]
				)
			)
			link.send(na)

	link.send(ereq)
	t.start(timer1)

	link.receive ( SixLowpanIPv6()/IPv6( src=nut.addr , dst = lbr.addr  )/ICMPv6EchoReply() )

	t.stop()

	log ("Common Test setup 1.1 terminated")



################################################################################
# CommonTestSetup_1_1b
################################################################################

def CommonTestSetup_1_1b( lbr , nut , prfx , timer1, link,  routerlt = 60 , prefixlt = 60 , contextlt = 60):
	#pass
	log ("Entering common test setup 1.1b")
	t = Timer()

	unexpected_packets.activate( link )
	timeout.activate( t )

	ra = pack (
		IPv6( src = lbr.lladdr , dst = nut.lladdr ) ,
		ICMPv6RAdv( rlt = routerlt, opt = [
			ICMPv6SLL( hw=lbr.hwaddr ) ,
			SixPI( pf=prfx , vlt = prefixlt ) ,
			SixCO( c=1 , clen=64, pf=prfx , lt=contextlt)
			]
		)
	)

	ereq = pack(
		IPv6 (src = lbr.addr, dst = nut.addr ),
		ICMPv6EchoRequest(),
		"6lowpan-nd-common-test-setup-1.1b"
	)

	t.start(timer1)

	link.receive (
		IPv6( src = nut.lladdr , dst = IPV6_ALL_ROUTERS ) /
		ICMPv6RSol( )
	)

	link.send(ra)
	t.start(timer1)

	with alt:	#FIXME: removable
		@link.receive (
			IPv6( src = nut.addr , dst = lbr.lladdr ) /
			ICMPv6NSol( opt = superop ( [
				ICMPv6SLL( hw=nut.hwaddr ) ,
				SixARO()
				] )
			)
		)
		def _(value):
			na = pack (
				IPv6( src = lbr.lladdr , dst = nut.lladdr ) ,
				ICMPv6NAdv( opt = [
					SixARO( lt = value[IPv6][ICMPv6]["opt"][SixARO]["lt"] , st = 0 ,
						eui = value[IPv6][ICMPv6]["opt"][SixARO]["eui"])
					]
				)
			)
			link.send(na)

	link.send(ereq)
	t.start(timer1)

	link.receive ( SixLowpanIPv6()/IPv6( src=nut.addr , dst = lbr.addr  )/ICMPv6EchoReply() )

	log ("Common Test setup 1.1b terminated")


################################################################################
# CommonTestSetup_1_2
################################################################################

def CommonTestSetup_1_2( lbr , nut , prfx , timer1, link,  routerlt = 60 , prefixlt = 60 , contextlt = 60):
	#pass

	log ("Entering common test setup 1.2")
	t = Timer()

	unexpected_packets.activate( link )
	timeout.activate( t )

	ra = pack (
		IPv6( src = lbr.lladdr , dst = nut.lladdr ) ,
		ICMPv6RAdv( rlt = routerlt, opt = [
				ICMPv6SLL( hw=lbr.hwaddr ),
				SixABRO(lbr=lbr.addr) ,
				SixPI( pf=prfx , vlt = prefixlt )
			]
		)
	)


	ereq = pack(
		IPv6 (src = lbr.addr, dst = nut.addr ),
		ICMPv6EchoRequest(),
		"6lowpan-nd-common-test-setup-1.2"
	)

	t.start(timer1)
	link.receive (
		IPv6( src = nut.lladdr , dst = IPV6_ALL_ROUTERS ) /
		ICMPv6RSol( )
	)
	link.send(ra)



	t.start(timer1)
	with alt:	#FIXME: removable
		@link.receive (
			IPv6( src = nut.addr , dst = lbr.lladdr ) /
			ICMPv6NSol( tgt =lbr.lladdr, opt = superop ( [
				ICMPv6SLL( hw = nut.hwaddr ) ,
				SixARO(eui=nut.hwaddr)
				] )
			)
		)
		def _(value):
			na = pack (
				IPv6( src = lbr.lladdr , dst = nut.lladdr ) ,
				ICMPv6NAdv( opt = [
					SixARO( lt = value[IPv6][ICMPv6]["opt"][SixARO]["lt"] , st = 0 )
					]
				)
			)
			link.send(na)

	t.start(timer1)
	link.send(ereq)
	link.receive ( SixLowpanIPv6()/IPv6( src=nut.addr , dst = lbr.addr  )/ICMPv6EchoReply() )
	t.stop()

	log ("Common Test setup 1.2 terminated")


################################################################################
# CommonTestSetup_1_2b
################################################################################

def CommonTestSetup_1_2b( lbr , nut , prfx , timer1, link,  routerlt = 60 , prefixlt = 60 , contextlt = 60):
	#pass

	log ("Entering common test setup 1.2b")
	t = Timer()

	unexpected_packets.activate( link )
	timeout.activate( t )

	ra = pack (
		IPv6( src = lbr.lladdr , dst = nut.lladdr ) ,
		ICMPv6RAdv( rlt = routerlt,
			opt = [
				SixABRO(lbr=lbr.addr) ,
				SixCO(lt =contextlt , clen = 1 ,c = 1, pf=prfx),
			]
		)
	)

	na = pack (
		IPv6( src = lbr.lladdr , dst = nut.lladdr ) ,
		ICMPv6NAdv( opt = [
			ICMPv6SLL( hw=lbr.hwaddr ) ,
			SixARO( lt= contextlt , st = 0 )
			]
		)
	)

	ereq = pack(
		IPv6 (src = lbr.addr, dst = nut.addr ),
		ICMPv6EchoRequest(),
		"6lowpan-nd-common-test-setup-1.2b"
	)

	t.start(timer1)

	link.receive (
		IPv6( src = nut.addr , dst = IPV6_ALL_ROUTERS ) /
		ICMPv6RSol( )
	)

	link.send(ra)
	t.start(timer1)

	with alt:	#FIXME: removable
		@link.receive (
			IPv6( src = nut.addr , dst = lbr.lladdr ) /
			ICMPv6NSol( opt = superop ( [
				ICMPv6SLL( hw = nut.hwaddr ) ,
				SixARO()
				] )

			)
		)
		def _(value):
			na = pack (
				IPv6( src = lbr.lladdr , dst = nut.lladdr ) ,
				ICMPv6NAdv( opt = [
					SixARO( lt = value[IPv6][ICMPv6]["opt"][SixARO]["lt"] , st = 0 )
					]
				)
			)
			link.send(na)

	link.send(ereq)
	t.start(timer1)

	link.receive ( SixLowpanIPv6()/IPv6( src=nut.addr , dst = lbr.addr  )/ICMPv6EchoReply() )

	log ("Common Test setup 1.2b terminated")

################################################################################
# CommonTestSetup_1_3
################################################################################
#TODO
def CommonTestSetup_1_3( host , nut , prfx , timer1, link,  routerlt = 60 , prefixlt = 60):
	#pass
	
	log ("Entering common test setup 1.3")
	t = Timer()

	unexpected_packets.activate( link )
	timeout.activate( t )

	log ("Common Test setup 1.3 terminated")


################################################################################
# CommonTestSetup_1_3b
################################################################################
#TODO
def CommonTestSetup_1_3b( host , nut , prfx , timer1, link,  routerlt = 60 , prefixlt = 60 , contextlt = 60):
	#pass
	log ("Entering common test setup 1.3b")
	t = Timer()

	unexpected_packets.activate( link )
	timeout.activate( t )

	log ("Common Test setup 1.3b terminated")


################################################################################
# CommonTestSetup_1_4
################################################################################

def CommonTestSetup_1_4(lbr , host,nut, prfx , timer1, link,  routerlt = 60 , prefixlt = 60, contextlt = 60  ):
	#TODO To modified to know if :
		# - the NUT is a LBR -> CommonTestSetup_1_3()
		# - the NUT is a LR -> CommonTestSetup_1_2()

	log ("Entering common test setup 1.4")
	t = Timer()

	unexpected_packets.activate( link )
	timeout.activate( t )

	#if nut==lbr:
	#	CommonTestSetup_1_3()

	#elif nut ==LR1 :
	#CommonTestSetup_1_2(lbr , nut , prfx , timer1, link,  routerlt, prefixlt, contextlt)

	log ("Common Test setup 1.4 terminated")


################################################################################
# CommonTestSetup_1_4b
################################################################################

def CommonTestSetup_1_4b(lbr , host ,nut, prfx , timer1, link,  routerlt = 60 , prefixlt = 60 , contextlt = 60 ):
	#TODO To modified to know if :
		# - the NUT is a LBR -> CommonTestSetup_1_3b()
		# - the NUT is a LR -> CommonTestSetup_1_2b()

	log ("Entering common test setup 1.4b")
	t = Timer()

	unexpected_packets.activate( link )
	timeout.activate( t )

	#if nut==lbr:
	#	CommonTestSetup_1_3b()

	#elif nut ==LR1 :
	#	CommonTestSetup_1_2b(lbr , nut , prfx , timer1, link,  routerlt, prefixlt, contextlt)


	log ("Common Test setup 1.4b terminated")

#########################################################################
# End # End # End # End # End # End # End # End # End # End # End # End #	
#########################################################################



from ..common import *
from ...core.templates import Range

class TD_COAP_BLOCK_01(CoAPTestCase):

    """
---
TD_COAP_BLOCK_01:
    obj: Handle GET blockwise transfer for large resource (early negotiation)
    cfg: CoAP_CFG_BASIC
    ref: '[BLOCK] 2.2-2.4'

    pre:
        - Client supports Block2 transfers
        - Server supports Block2 transfers
        - Server offers a large resource /large
        - Client knows /large requires block transfer

    seq:
    -   s: "Client is requested to retrieve resource /large"

    -   c:
        - 'Client sends a GET request. The request optionally contains
        a Block2 option indicating:'
        -   - NUM = 0;
            - M = 0;
            - SZX = the desired block size.

    -   c:
    - 'Server sends 2.05 (Content) response with a Block2 option
    indicating:'
    -   - NUM = 0;
        - M = 1;
        - "SZX is less or equal to the desired block size indicated
        by the GET request"
        - Payload size is 2SZX+4 bytes.

    -   c:
        - 'Client send GET requests for further blocks indicating:'
        -   - 'NUM = i where “i” is the block number of the current
        block'
            - M = 0
            - SZX is the SZX at step 3

    -   c:
        - 'Server sends 2.05 (Content) response containing Block2
        option indicating:'
        -   - NUM = i where “i” is the block number used at step 4
            - M = 1
            - SZX is the SZX at step 3
            - Payload size MUST be 2SZX+4 bytes

    -   c:
    - 'Client send GET request for the last block indicating:'
    -   - NUM = n where “n” is the last block number
        - M = 0
        - SZX is the SZX at step 3

    -   c:
        - 'Server sends 2.05 (Content) response with a Block2 option
        indicating:'
        -   - NUM = n where “n” is the block number used at step 6;
            - M = 0;
            - SZX is the SZX at step 3.
            - Payload size is lesser or equal to 2SZX+4 bytes.

    -   v: 'Client displays the received information'

    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='con', code='get', opt=Opt(CoAPOptionUriPath("large"),CoAPOptionBlock2())),
        ]

    def run (self):
        # Step 2 - initial request from the client
        self.match("client", CoAP (code = "get",
                         opt = self.uri ("/large", CoAPOptionBlock2(num=0))))
        self.match("client", CoAP (opt = Opt (CoAPOptionBlock2(m=0))), "fail")
        client_szx  = self.coap["opt"][CoAPOptionBlock2]["szx"]

        self.next_skip_ack()

        # Step 3 - response from the server w/ block size negociation
        if not self.match("server", CoAP (code = 2.05,
                         opt = Opt (CoAPOptionBlock2 ())
                )):
            raise self.Stop()
        self.match("server", CoAP (opt = Opt (CoAPOptionBlock2 (num=0, szx=Range(int, 0, client_szx)))
                ), "fail")

        server_szx  = self.coap["opt"][CoAPOptionBlock2]["szx"]
        server_size = 2**(server_szx+4)

        # Step 3 - expect more blocks
        self.match("server", CoAP (opt = Opt (CoAPOptionBlock2(m=1)),
                         pl = Length (bytes, server_size)
                ))

        # Step 4 - 5
        for i in itertools.count(1):
            self.next_skip_ack()

            # Step 4 - request next block from client
            # (This is Step 6 if last block.)
            if self.match("client", CoAP (code = "get",
                             opt = Opt (CoAPOptionBlock2 (num=i, szx=server_szx)))):
                self.match("client", CoAP (opt = Opt (CoAPOptionBlock2 (m=0))), "fail")

            self.next_skip_ack()

            # Step 5 - send next block from server
            if not self.match("server", CoAP (code = 2.05,
                             opt = Opt (CoAPOptionBlock2 (num=i, szx=server_szx))
                    )):
                break

            if not self.match("server", CoAP (opt = Opt (CoAPOptionBlock2 (m=1)),
                                pl = Length (bytes, server_size)
                        ), None):
                # Step 7 - no more blocks
                self.match("server", CoAP (opt = Opt (CoAPOptionBlock2 (m=0)),
                                pl = Length (bytes, (1, server_size))
                        ), "fail")

                # end of testcase
                self.next_skip_ack(optional = True)
                break



import unittest
from ttproto.utils.version_git import get_git_version


class VersionTestCase(unittest.TestCase):

    def test_not_none(self):
        self.assertIsNotNone(get_git_version(), "git version shouldnt be none")

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
#  (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import base64
import pika
import time
import hashlib
import json
import errno
import os
import sys
import uuid
import signal
import logging

from collections import OrderedDict

# Directories
from ttproto import DATADIR
from ttproto import TMPDIR
from ttproto import LOGDIR

from ttproto import LOG_LEVEL
from ttproto.tat_services import analyze_capture, dissect_capture, get_test_cases, base64_to_pcap_file

from ttproto.core.typecheck import typecheck, optional, either
from ttproto.utils import pure_pcapy
from ttproto.utils.rmq_handler import AMQP_URL, AMQP_EXCHANGE, JsonFormatter, RabbitMQHandler
from ttproto.utils import messages
from ttproto.utils.packet_dumper import launch_amqp_data_to_pcap_dumper, AmqpDataPacketDumper
from ttproto.utils import event_bus_utils

COMPONENT_ID = NotImplementedError

AUTO_DISSECT_OUTPUT_FILE = os.path.join(DATADIR, 'auto_dissection')

# Prefix and suffix for the hashes
HASH_PREFIX = 'tt'
HASH_SUFFIX = 'proto'
TOKEN_LENGTH = 28

# states
previous_frames_count = 0

logger = logging.getLogger(__name__)


#####################


def launch_tat_amqp_interface(amqp_url, amqp_exchange, tat_protocol, dissection_auto):
    """
    Att this is blocking. Launches the TAT AMQP interface.
    See doc for more info about the AMQP API endpoints
    """

    def signal_int_handler(self, frame):
        print('got SIGINT, stopping dumper..')

        if amqp_interface:
            amqp_interface.stop()

    signal.signal(signal.SIGINT, signal_int_handler)
    amqp_interface = AmqpInterface(amqp_url, amqp_exchange, tat_protocol, dissection_auto)
    amqp_interface.run()


class AmqpInterface:
    def __init__(self, amqp_url, amqp_exchange, tat_protocol, dissection_auto):
        self.COMPONENT_ID = 'tat|amqp_interface'
        self.tat_protocol = tat_protocol
        self.dissection_auto = dissection_auto

        self.logger = logging.getLogger(self.COMPONENT_ID)
        self.logger.setLevel(LOG_LEVEL)

        # AMQP log handler & formatter
        rabbitmq_handler = RabbitMQHandler(AMQP_URL, self.COMPONENT_ID)
        json_formatter = JsonFormatter()
        rabbitmq_handler.setFormatter(json_formatter)
        self.logger.addHandler(rabbitmq_handler)

        self.amqp_url = amqp_url
        self.amqp_exchange = amqp_exchange
        self.connection = pika.BlockingConnection(pika.URLParameters(self.amqp_url))
        self.channel = self.connection.channel()

        # init AMQP BUS communication vars

        self.services_queue_name = 'services_queue@%s' % self.COMPONENT_ID
        self.channel.queue_declare(queue=self.services_queue_name,
                                   auto_delete=True,
                                   arguments={'x-max-length': 100})

        # subscribe to analysis services requests
        self.channel.queue_bind(exchange=AMQP_EXCHANGE,
                                queue=self.services_queue_name,
                                routing_key=messages.MsgInteropTestCaseAnalyze.routing_key)

        # subscribe to dissection services requests
        self.channel.queue_bind(exchange=AMQP_EXCHANGE,
                                queue=self.services_queue_name,
                                routing_key=messages.MsgDissectionDissectCapture.routing_key)

        self.channel.basic_qos(prefetch_count=1)
        self.channel.basic_consume(self.on_service_request, queue=self.services_queue_name)

        if self.dissection_auto:
            self.data_queue_name = 'data_plane_messages@%s' % self.COMPONENT_ID
            self.channel.queue_declare(queue=self.data_queue_name,
                                       auto_delete=True,
                                       arguments={'x-max-length': 100})

            self.channel.basic_qos(prefetch_count=1)
            self.channel.basic_consume(self.on_data_received, queue=self.data_queue_name)

            # subscribe to data events just to check if there's activity in the data plane
            self.channel.queue_bind(exchange=AMQP_EXCHANGE,
                                    queue=self.data_queue_name,
                                    routing_key='fromAgent.#.packet.raw')

    def run(self):
        # let's send bootstrap message (analysis)
        event_bus_utils.publish_message(
            self.connection,
            messages.MsgTestingToolComponentReady(component='analysis')
        )

        #  let's send bootstrap message (dissector)
        event_bus_utils.publish_message(
            self.connection,
            messages.MsgTestingToolComponentReady(component='dissection')
        )

        # start main job (the following is a blocking call)

        self.logger.info("Awaiting for analysis & dissection requests")
        self.channel.start_consuming()

    def stop(self):

        # FINISHING... let's send a goodby message
        if self.channel is None:
            self.channel = self.connection.channel()

        # dissection shutdown message
        event_bus_utils.publish_message(
            self.connection,
            messages.MsgTestingToolComponentShutdown(component='dissection')
        )

        # analysis shutdown message
        event_bus_utils.publish_message(
            self.connection,
            messages.MsgTestingToolComponentShutdown(component='analysis')
        )

        self.connection.close()

        self.logger.info('Stopping.. Bye bye!')

        sys.exit(0)

    def on_data_received(self, ch, method, props, body):
        global previous_frames_count
        ch.basic_ack(delivery_tag=method.delivery_tag)

        try:
            event_received = messages.Message.load_from_pika(method, props, body)
        except Exception as e:
            self.logger.error(str(e))
            return

        if isinstance(event_received, messages.MsgPacketSniffedRaw):

            try:
                if 'serial' in event_received.interface_name:
                    pcap_to_dissect = os.path.join(AmqpDataPacketDumper.DEFAULT_DUMP_DIR,
                                                   AmqpDataPacketDumper.DEFAULT_802154_DUMP_FILENAME
                                                   )
                elif 'tun' in event_received.interface_name:
                    pcap_to_dissect = os.path.join(AmqpDataPacketDumper.DEFAULT_DUMP_DIR,
                                                   AmqpDataPacketDumper.DEFAULT_RAWIP_DUMP_FILENAME
                                                   )
                else:
                    self.logger.error('Not implemented protocol dissection for %s' % event_received.interface_name)
                    return

                self.logger.info("Data plane activity")

                # note : the sniffing and pcap generation is handled by another process (packet dumper component)
                ch.queue_purge(queue=self.data_queue_name)

                dissection_structured_text, dissection_simple_text = dissect_capture(
                    filename=pcap_to_dissect,
                    proto_filter=None,
                    output_filename=AUTO_DISSECT_OUTPUT_FILE,
                    number_of_frames_to_skip=previous_frames_count
                )
                previous_frames_count += len(dissection_structured_text)

            except (TypeError, pure_pcapy.PcapError) as e:
                self.logger.error("Error processing PCAP: %s" % e)
                return

            except Exception as e:
                self.logger.error(str(e))
                return

            # prepare response with dissection info:
            event_diss = messages.MsgDissectionAutoDissect(
                token=None,
                frames=dissection_structured_text,
                frames_simple_text=dissection_simple_text,
                testcase_id=None,
                testcase_ref=None,
            )
            event_bus_utils.publish_message(self.connection, event_diss)
            self.logger.info("Auto dissection message sent (%s frames).. " % len(dissection_structured_text))

            return

        else:
            self.logger.debug('Unknonwn message. Message dropped: %s' % event_received)

    def on_service_request(self, ch, method, props, body):
        ch.basic_ack(delivery_tag=method.delivery_tag)

        try:
            service_request = messages.Message.load_from_pika(method, props, body)
        except Exception as e:
            self.logger.error(str(e))
            return

        if isinstance(service_request, messages.MsgInteropTestCaseAnalyze):
            self.logger.debug("Starting analysis of PCAP")

            # generation of token
            operation_token = _get_token()

            try:
                pcap_file_base64 = service_request.value
                filename = service_request.filename
                testcase_id = service_request.testcase_id
                testcase_ref = service_request.testcase_ref
                protocol = self.tat_protocol
                if hasattr(service_request, 'protocol') and service_request.protocol is not None:
                    protocol = service_request.protocol

                nb = base64_to_pcap_file(os.path.join(TMPDIR, filename), pcap_file_base64)

                # if pcap file has less than 24 bytes then its an empty pcap file
                if (nb <= 24):
                    event_bus_utils.publish_message(
                        self.connection,
                        messages.MsgErrorReply(
                            service_request,
                            ok=False,
                            error_code=400,
                            error_message='Empty PCAP file received'
                        )
                    )
                    self.logger.warning("Empty PCAP received")
                    return
                else:
                    self.logger.info("Pcap correctly saved %d B at %s" % (nb, os.path.join(TMPDIR, filename)))

                # run the analysis
                analysis_results = analyze_capture(
                    filename=os.path.join(TMPDIR, filename),
                    testcase_id=testcase_id,
                    protocol=protocol,
                    output_filename=os.path.join(DATADIR, operation_token),
                )

            except Exception as e:
                event_bus_utils.publish_message(
                    self.connection,
                    messages.MsgErrorReply(
                        service_request,
                        error_message=str(e)
                    )
                )
                self.logger.error(e)
                self.logger.error(type(e))
                return

            # let's prepare the message
            try:
                response = messages.MsgInteropTestCaseAnalyzeReply(
                    service_request,
                    ok=True,
                    verdict=analysis_results[1],
                    description=analysis_results[3],
                    review_frames=analysis_results[2],
                    partial_verdicts=analysis_results[4],
                    token=operation_token,
                    testcase_id=testcase_id,
                    testcase_ref=testcase_ref
                )
                # send response
                event_bus_utils.publish_message(self.connection, response)
                self.logger.info("Analysis response sent")

            except Exception as e:
                event_bus_utils.publish_message(
                    self.connection,
                    messages.MsgErrorReply(
                        service_request,
                        error_message=str(e)
                    )
                )
                self.logger.error(str(e))
                return

        elif isinstance(service_request, messages.MsgTestSuiteGetTestCases):
            self.logger.warning("API call not implemented. Test coordinator provides this service.")
            return

            # # Get the list of test cases
            # try:
            #     test_cases = get_test_cases()
            #
            #     # lets prepare content of response
            #     tc_list = []
            #     for tc in test_cases:
            #         tc_list.append(test_cases[tc]['tc_basic'])
            #     #TODO build & send response
            # except FileNotFoundError as fnfe:
            #     _publish_message(
            #             ch,
            #             amqp_messages.MsgErrorReply(ok=False, error_message='File not found error')
            #     )
            #     self.logger.error('Cannot fetch test cases list:\n' + str(fnfe))
            #     return

        elif isinstance(service_request, messages.MsgDissectionDissectCapture):
            self.logger.info("Starting dissection of PCAP ...")
            self.logger.info("Decoding PCAP file using base64 ...")

            # get dissect params from request
            pcap_file_base64 = service_request.value
            filename = service_request.filename
            proto_filter = service_request.protocol_selection

            # save pcap as file
            nb = base64_to_pcap_file(os.path.join(TMPDIR, filename), pcap_file_base64)

            # if pcap file has less than 24 bytes then its an empty pcap file
            if (nb <= 24):
                event_bus_utils.publish_message(
                    self.connection,
                    messages.MsgErrorReply(
                        service_request,
                        error_code=400,
                        error_message='Empty PCAP file received'
                    )
                )
                self.logger.warning("Empty PCAP received")
                return

            else:
                self.logger.info("Pcap correctly saved %d B at %s" % (nb, TMPDIR))

            # Lets dissect
            try:
                operation_token = _get_token()
                dissection_structured_text, dissection_simple_text = dissect_capture(
                    filename=os.path.join(TMPDIR, filename),
                    proto_filter=proto_filter,
                    output_filename=os.path.join(DATADIR, operation_token),
                )
            except (TypeError, pure_pcapy.PcapError) as e:
                event_bus_utils.publish_message(
                    self.connection,
                    messages.MsgErrorReply(
                        service_request,
                        error_message="Error processing PCAP. Error: %s" % str(e)
                    )
                )
                self.logger.error("Error processing PCAP: %s" % e)
                return
            except Exception as e:
                event_bus_utils.publish_message(
                    self.connection,
                    messages.MsgErrorReply(
                        service_request,
                        error_message="Error found while dissecting pcap. Error: %s" % str(e)
                    )
                )
                self.logger.error(str(e))
                return

            # prepare response with dissection info:
            response = messages.MsgDissectionDissectCaptureReply(
                service_request,
                token=operation_token,
                frames=dissection_structured_text,
                frames_simple_text=dissection_simple_text
            )
            event_bus_utils.publish_message(self.connection, response)
            return

        else:
            self.logger.warning('Coudnt process the service request: %s' % service_request)
            return


# # # AUXILIARY FUNCTIONS # # #

def _auto_dissect_service():
    global AUTO_DISSECT_PERIOD
    last_polled_pcap = None

    # setup process own connection
    connection = pika.BlockingConnection(pika.URLParameters(AMQP_URL))

    while True:
        time.sleep(AUTO_DISSECT_PERIOD)

        logger.debug('Entering auto triggered dissection process')

        # request to sniffing component
        try:
            request = messages.MsgSniffingGetCaptureLast()
            response = event_bus_utils.amqp_request(connection, request, COMPONENT_ID, use_message_typing=True)

        except event_bus_utils.AmqpSynchCallTimeoutError as amqp_err:
            logger.error(
                'Sniffer didnt respond to Request: %s . Error: %s'
                % (
                    type(request),
                    str(amqp_err)
                )
            )
            return

        if response.ok is False:
            logger.error(
                'Sniffing component coundlt process the %s request correcly, response: %s'
                % (
                    type(request),
                    repr(request)
                )
            )
        else:
            if last_polled_pcap and last_polled_pcap == response.value:
                logger.debug('No new sniffed packets to dissect')
            else:
                logger.debug("Starting auto triggered dissection.")

                # get dissect params from request
                pcap_file_base64 = response.value
                filename = response.filename
                proto_filter = None

                last_polled_pcap = pcap_file_base64

                # save pcap as file
                nb = base64_to_pcap_file(os.path.join(TMPDIR, filename), pcap_file_base64)

                # if pcap file has less than 24 bytes then its an empty pcap file
                if (nb <= 24):
                    logger.warning("Empty PCAP received received.")

                else:
                    logger.info("Pcap correctly saved %d B at %s" % (nb, TMPDIR))

                    # let's dissect
                    try:
                        operation_token = _get_token()
                        dissection_structured_text, dissection_simple_text = dissect_capture(
                            filename=os.path.join(TMPDIR, filename),
                            proto_filter=proto_filter,
                            output_filename=os.path.join(DATADIR, operation_token)
                        )
                    except (TypeError, pure_pcapy.PcapError) as e:
                        logger.error("Error processing PCAP. More: %s" % str(e))
                        return
                    except Exception as e:
                        logger.error("Error while dissecting. Error: %s" % str(e))
                        return

                    # lets create and push the message to the bus
                    m = messages.MsgDissectionAutoDissect(
                        token=operation_token,
                        frames=dissection_structured_text,
                        frames_simple_text=dissection_simple_text,
                        testcase_id=filename.strip('.pcap'),  # dirty solution but less coding :)
                        testcase_ref=None,
                    )
                    event_bus_utils.publish_message(connection, m)


@typecheck
def _get_token(tok: optional(str) = None):
    """
    Function to get a token, if there's a valid one entered just return it
    otherwise generate a new one

    :param tok: The token if there's already one
    :type tok: str

    :return: A token, the same if there's already one, a new one otherwise
    :rtype: str
    """

    # If the token is already a correct one
    try:
        if all((
                tok,
                type(tok) == str,
                len(tok) == 28,
                base64.urlsafe_b64decode(tok + '=')  # Add '=' only for checking
        )):
            return tok
    except:  # If the decode throw an error => Wrong base64
        pass

    # Generate a token
    token = hashlib.sha1(
        str.encode((
            "%s%04d%s" %
            (
                HASH_PREFIX,
                time.time(),
                HASH_SUFFIX
            )
        ), encoding='utf-8')
    )
    token = base64.urlsafe_b64encode(token.digest()).decode()

    # Remove the '=' at the end of it, it is used by base64 for padding
    return token.replace('=', '')



#!/usr/bin/env python3
#
#  (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

import re

from .templates import *
from ttproto.core.analyzer import TestCase, is_protocol, Node, Conversation, Capture
from ttproto.core.dissector import Frame
from ttproto.core.templates import All, Not, Any, Length
from ttproto.core.typecheck import *
from ttproto.core.lib.all import *
from urllib import parse
from ttproto.core.exceptions import Error
from ttproto.tat_coap.common import CoAPTestCase, NoStimuliFoundForTestcase, FilterError

from xml.etree import ElementTree
import logging
import requests
import json
import sys

# CoAP constants
RESPONSE_TIMEOUT = 2
RESPONSE_RANDOM_FACTOR = 1.5
MAX_RETRANSMIT = 4
MAX_TIMEOUT = 10 + round(
    (RESPONSE_TIMEOUT * RESPONSE_RANDOM_FACTOR) * 2 ** MAX_RETRANSMIT
)


def validate(jsond, objectid):
    logging.info("ObjectID: " + objectid)
    jsondata = jsond[2:]
    jsondata = jsondata[:-1]
    jsondata = json.loads(jsondata)
    # logging.info json.dumps(jsondata, sort_keys=True,indent=4, separators=(',', ': '))

    url = 'http://www.openmobilealliance.org/api/lwm2m/v1/Object?ObjectID=' + objectid;
    logging.info("Registry url: " + url)
    r = requests.get(url)
    # logging.info json.dumps(r.json(), sort_keys=True,indent=4, separators=(',', ': '))
    j = r.json()
    logging.info("ObjectLink: " + j[0]["ObjectLink"])
    r2 = requests.get(j[0]["ObjectLink"])
    # logging.info r2.content
    root = ElementTree.fromstring(r2.content)
    
    result = []
    validation = "pass"
    message = "["
    for item in root.iter('Item'):
        for child in item:
            if (child.text == "Mandatory"):
                if (item.find("Operations").text != "E"):
                    logging.info("ResourceID: " + item.get("ID"))
                    logging.info("Name: " + str(item.find("Name").text))
                    logging.info("Type: " + str(item.find("Type").text))
                    logging.info("Operations: " + str(item.find("Operations").text))
                    logging.info("MultipleInstances: " + str(item.find("MultipleInstances").text))
                    for attrs in jsondata['e']:
                        itemuri = item.get("ID");
                        if (item.find("MultipleInstances").text == "Multiple"):
                            itemuri = itemuri + "/0";
                        if attrs['n'] == itemuri:
                            n = attrs['n']
                            logging.info("Found!")
                            message+="{"+str(item.find("Name").text)+" : validated}"
                            break
                    else:
                        validation = "fail"
                        logging.info('Not found!')
                        message+="{"+str(item.find("Name").text)+" : not validated}"
    
    message += "]"
    result.append(validation)
    result.append(message)
    logging.info("Validation: " + result[0])
    return result


if __name__ == "__main__":
    pass



from ..common import *


class TD_COAP_OBS_04(CoAPTestCase):
    """
---
TD_COAP_OBS_04:
    obj: Client detection of deregistration (Max-Age)
    cfg: CoAP_CFG_BASIC
    ref: '[OBSERVE] 3.3.1 §4'

    pre:
        - Client supports Observe option
        - Server supports Observe option
        - "Server offers an observable resource /obs which changes periodically
           (e.g. every 5s) which produces confirmable notifications"
    seq:
    -   s: "Client is requested to send to the server a confirmable GET
            request with observe option for resource /obs"

    -   c:
        - 'The request sent by client contains:'
        -   - Type = 0 (CON)
            - Code = 1 (GET)
            - Token value  = a value generated by the client
            - Observe option = empty

    -   c:
        - 'Server sends the response containing:'
        -   - Type = 2 (ACK)
            - Code = 2.05 (Content)
            - Content-format of the resource /obs
            - Token value = same as one found in the step 2
            - Observe option with a sequence number

    -   c:
        - 'Server sends a notification containing:'
        -   - Type = 0 (CON)
            - Code = 2.05 (Content)
            - Content-format = same as one found in the step 3
            - Token value = same as one found in the step 2
            - Observe option indicating increasing values

    -   v: Client displays the received information

    -   c: Client sends an ACK

    -   s: Server is rebooted or in another way caused to lose its observation state

    -   c:
        - Server does not send notifications

    -   v: Client does not display updated information

    -   v:After a while (see note) the client internally decides to send another GET request to the server with observe option for resource /obs, using Token t again to confirm the registration

    -   v:
    - 'Client sends a GET request to the server for resource /obs:'
    -   - Type = 0(CON)
        - Code = 1(GET)
        - Token value  = same as one found in the step 2
        - Observe option = empty

    -   c:
        - 'Server sends the response containing:'
        -   -Type = 2(ACK)
            - Code = 2.05(Content)
            - Content-format of the resource /obs
            - Token value = same as one found in the step 2
            - Observe option with a sequence number that is not necessarily increasing

    -   c:
        - 'Server sends a notification containing:'
        -   - Type = 0(CON)
            - Code = 2.05(Content)
            - Content-format = same as one found in the step 12
            - Token value = same as one found in the step 2
            - Observe option indicating increasing values
    -   v: Client displays the received information

    -   c: Client sends an ACK

    """
    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
                CoAP(type='con', code='get', opt=Opt(CoAPOptionObserve(0),CoAPOptionUriPath("obs")))
        ]

    def run(self):
                request = CoAP(type="con", code="get",	opt=self.uri("/obs", CoAPOptionObserve(0)))
                response = CoAP(type="ack", code=2.05, opt=Opt(CoAPOptionObserve(), CoAPOptionContentFormat()))

                # Step 2
                self.match("client", request)

                token = self.coap["tok"]
                uri = self.coap.get_uri()

                self.next()

                # Step 3
                if not self.match("server", response):
                    raise self.Stop()
                self.match("server", CoAP(tok = token), "fail")

                content_format = self.coap["opt"][CoAPOptionContentFormat]["val"]
                index          = self.coap["opt"][CoAPOptionObserve]["val"]

                self.next()

                # we need to observe at least one notification
                # (to ensure that the server is observing the resource for the client)

                verdict_if_none = "inconclusive"
                log_message_if_not_pass = "Test case did not encounter any re-registration (STEP 10).\
It may be because the server did not lost it's observation state as expected in step 7,\
or because the client did not try to re-register."
                client_did_re_register = False
                client_received_notification_after_reregistering = False
                # Step 4
                while self.match("server", CoAP(type="con", code=2.05, opt=Opt(CoAPOptionObserve())), verdict_if_none, log_message_if_not_pass):
                    self.match("server", CoAP(tok = token), "fail")
                    self.match("server", CoAP(opt = Opt(CoAPOptionContentFormat(content_format))), "fail")

                    new_index = self.coap["opt"][CoAPOptionObserve]["val"]

                    self.set_verdict(("pass" if new_index > index else "fail"),
                    "value of observe option must be increasing")

                    self.next()

                    # Step 6
                    self.match("client",CoAP(type="ack", code=0), verdict_if_none, log_message_if_not_pass)
                    # We pass client_received_notification_after_reregistering to
                    # as argument as if it's true, it may be the end of the
                    # conversation anytime.
                    # It avoid having a wrong inconclusive verdict
                    # with "premature end of conversation" when it should
                    # be a pass verdict.
                    self.next(client_received_notification_after_reregistering)
                    if client_did_re_register:
                        # Step 13
                        client_received_notification_after_reregistering = True
                        verdict_if_none = None
                    else:
                        client_did_re_register = self.__client_re_registred_with_same_token(token)
                        skiped_all_retry = False
                        while not skiped_all_retry and client_did_re_register:
                            # We do this loop because the client may do several
                            # retries before being successfully registred again.
                            retry = self.__client_re_registred_with_same_token(token)
                            skiped_all_retry = not retry
                        if client_did_re_register:
                            # Step 12
                            self.match("server",
                                        CoAP(type="ack", tok=token, code=2.05, opt=Opt(CoAPOptionObserve(),CoAPOptionContentFormat())), "fail",
                                        "Client did re_register as expected, but the server did not send new notification")
                            index = self.coap["opt"][CoAPOptionObserve]["val"]
                            self.next()

    def __client_re_registred_with_same_token(self, token):
        re_registered = self.match("client",CoAP(type='con', code='get',
        tok=token, opt=Opt(CoAPOptionObserve(0),CoAPOptionUriPath("obs"))), None)
        if re_registered:
            self.next()
        return re_registered



#
#   (c) 2012  Universite de Rennes 1
#
# Contact address: <t3devkit@irisa.fr>
#
#
# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".
#
# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.
#
# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.
#
# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
from	ttproto.core.data		import BidictValueType

# default NH will be 59 (IPv6-NoNxt)
ip_next_header_bidict = BidictValueType (59, bytes)

# source:	http://www.iana.org/assignments/protocol-numbers/protocol-numbers.txt
ip_next_header_descriptions = {
    0:	"IPv6 Hop-by-Hop Option",
    1:	"Internet Control Message",
    2:	"Internet Group Management",
    3:	"Gateway-to-Gateway",
    4:	"IPv4 encapsulation",
    5:	"Stream",
    6:	"Transmission Control",
    7:	"CBT",
    8:	"Exterior Gateway Protocol",
    9:	"any private interior gateway (used by Cisco for their IGRP)",
    10:	"BBN RCC Monitoring",
    11:	"Network Voice Protocol",
    12:	"PUP",
    13:	"ARGUS",
    14:	"EMCON",
    15:	"Cross Net Debugger",
    16:	"Chaos",
    17:	"User Datagram",
    18:	"Multiplexing",
    19:	"DCN Measurement Subsystems",
    20:	"Host Monitoring",
    21:	"Packet Radio Measurement",
    22:	"XEROX NS IDP",
    23:	"Trunk-1",
    24:	"Trunk-2",
    25:	"Leaf-1",
    26:	"Leaf-2",
    27:	"Reliable Data Protocol",
    28:	"Internet Reliable Transaction",
    29:	"ISO Transport Protocol Class 4",
    30:	"Bulk Data Transfer Protocol",
    31:	"MFE Network Services Protocol",
    32:	"MERIT Internodal Protocol",
    33:	"Datagram Congestion Control Protocol",
    34:	"Third Party Connect Protocol",
    35:	"Inter-Domain Policy Routing Protocol",
    36:	"XTP",
    37:	"Datagram Delivery Protocol",
    38:	"IDPR Control Message Transport Proto",
    39:	"TP++ Transport Protocol",
    40:	"IL Transport Protocol",
    41:	"IPv6 encapsulation",
    42:	"Source Demand Routing Protocol",
    43:	"Routing Header for IPv6",
    44:	"Fragment Header for IPv6",
    45:	"Inter-Domain Routing Protocol",
    46:	"Reservation Protocol",
    47:	"General Routing Encapsulation",
    48:	"Dynamic Source Routing Protocol",
    49:	"BNA",
    50:	"Encap Security Payload",
    51:	"Authentication Header",
    52:	"Integrated Net Layer Security  TUBA",
    53:	"IP with Encryption",
    54:	"NBMA Address Resolution Protocol",
    55:	"IP Mobility",
    56:	"Transport Layer Security Protocol using Kryptonet key management",
    57:	"SKIP",
    58:	"ICMP for IPv6",
    59:	"No Next Header for IPv6",
    60:	"Destination Options for IPv6",
    61:	"any host internal protocol",
    62:	"CFTP",
    63:	"any local network",
    64:	"SATNET and Backroom EXPAK",
    65:	"Kryptolan",
    66:	"MIT Remote Virtual Disk Protocol",
    67:	"Internet Pluribus Packet Core",
    68:	"any distributed file system",
    69:	"SATNET Monitoring",
    70:	"VISA Protocol",
    71:	"Internet Packet Core Utility",
    72:	"Computer Protocol Network Executive",
    73:	"Computer Protocol Heart Beat",
    74:	"Wang Span Network",
    75:	"Packet Video Protocol",
    76:	"Backroom SATNET Monitoring",
    77:	"SUN ND PROTOCOL-Temporary",
    78:	"WIDEBAND Monitoring",
    79:	"WIDEBAND EXPAK",
    80:	"ISO Internet Protocol",
    81:	"VMTP",
    82:	"SECURE-VMTP",
    83:	"VINES",
    84:	"TTP",
    84:	"Protocol Internet Protocol Traffic Manager",
    85:	"NSFNET-IGP",
    86:	"Dissimilar Gateway Protocol",
    87:	"TCF",
    88:	"EIGRP",
    89:	"OSPFIGP",
    90:	"Sprite RPC Protocol",
    91:	"Locus Address Resolution Protocol",
    92:	"Multicast Transport Protocol",
    93:	"AX.25 Frames",
    94:	"IP-within-IP Encapsulation Protocol",
    95:	"Mobile Internetworking Control Pro.",
    96:	"Semaphore Communications Sec. Pro.",
    97:	"Ethernet-within-IP Encapsulation",
    98:	"Encapsulation Header",
    99:	"any private encryption scheme",
    100:	"GMTP",
    101:	"Ipsilon Flow Management Protocol",
    102:	"PNNI over IP",
    103:	"Protocol Independent Multicast",
    104:	"ARIS",
    105:	"SCPS",
    106:	"QNX",
    107:	"Active Networks",
    108:	"IP Payload Compression Protocol",
    109:	"Sitara Networks Protocol",
    110:	"Compaq Peer Protocol",
    111:	"IPX in IP",
    112:	"Virtual Router Redundancy Protocol",
    113:	"PGM Reliable Transport Protocol",
    114:	"any 0-hop protocol",
    115:	"Layer Two Tunneling Protocol",
    116:	"D-II Data Exchange (DDX)",
    117:	"Interactive Agent Transfer Protocol",
    118:	"Schedule Transfer Protocol",
    119:	"SpectraLink Radio Protocol",
    120:	"UTI",
    121:	"Simple Message Protocol",
    122:	"SM",
    123:	"Performance Transparency Protocol",
    126:	"Combat Radio Transport Protocol",
    127:	"Combat Radio User Datagram",
    130:	"Secure Packet Shield",
    131:	"Private IP Encapsulation within IP",
    132:	"Stream Control Transmission Protocol",
    133:	"Fibre Channel",
    138:	"MANET Protocols",
    139:	"Host Identity Protocol",
    140:	"Shim6 Protocol",
    141:	"Wrapped Encapsulating Security Payload",
    142:	"Robust Header Compression",
    253:	"Use for experimentation and testing",
    254:	"Use for experimentation and testing",
}



"""
Invokes webserver to be run at 127.0.0.1:2082
Should be run as: python3 -m ttproto.tat_6lowpan
"""
from .webserver import *

if __name__ == "__main__":

    reopen_log_file(None, None)

    __shutdown = False

    def shutdown():
        global __shutdown
        __shutdown = True

    for d in TMPDIR, DATADIR, LOGDIR:
        try:
            os.makedirs(d)
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise

    result = Analyzer('tat_privacy').analyse('./tests/test_dumps/coap/TD_COAP_CORE_01_PASS.pcap', 'TD_COAP_ANALYSIS_1')

    for res in result:
        print(res)
    if result[1] != 'pass' and len(result[5]) > 0:
        import traceback

        traceback.print_tb(result[5][0][2])
    shutdown()

def reopen_log_file(signum, frame):
    global log_file
    log_file = open(os.path.join(LOGDIR, "tat-6lowpan-webserver.log"), "a")


reopen_log_file(None, None)

# log rotation
# -> reopen the log file upon SIGHUP
signal.signal(signal.SIGHUP, reopen_log_file)

server = http.server.HTTPServer(("0.0.0.0", 2082), RequestHandler)
print('Server is ready')
while not __shutdown:
    try:
        l = log_file
        server.handle_request()
    except select.error:
        # do not abort when we receive a signal
        if l == log_file:
            raise

    if len(sys.argv) > 1:
        break



from ..common import *


class TD_LWM2M_1_INT_104(CoAPTestCase):
    """
---
testcase_id: TD_LWM2M_1.0_INT_104
uri : http://openmobilealliance.org/iot/lightweight-m2m-lwm2m
configuration: LWM2M_CFG_01
objective: Test that the Client updates its registration with the Server when triggered with the Registration Update Trigger.
pre_conditions:
  - Device is turned on. 
  - The bootstarp procedure has been completed or the required bootstarp information is available to the client.
  - The device is registred with the Server with a Lifetime of 20s. 
  - The Client supports the minimum Configuration C.1.
references: 'OMA-ETS-LightweightM2M-V1_0_1-20170926-A'
sequence:
  - step_id: 'TD_LWM2M_1.0_INT_104_step_01'
    type: stimuli
    node : lwm2m_server
    description:
      - 'Lwm2m server is requested to send Registration Update Tigger message (CoAP POST) to client'
      - - Type = 0 (CON)
        - Code = 2 (POST)

  - step_id: 'TD_LWM2M_1.0_INT_104_step_02'
    type: check
    description:
      - 'The request sent by the server contains'
      - - Type=0 and Code=2
        - Uri-Path option= 1/0/8

  - step_id: 'TD_LWM2M_1.0_INT_104_step_03'
    type: check
    description:
        - 'Lwm2m client sends response containing'
        - - Code = 2.04 (Changed)

  - step_id: 'TD_LWM2M_1.0_INT_104_step_04'
    type: verify
    node: lwm2m_server
    description:
        - 'Server receives a Success message'

  - step_id: 'TD_LWM2M_1.0_INT_104_step_05'
    type: stimuli
    node : lwm2m_client
    description:
      - 'LwM2M Client is requested to send an UPDATE Registration message without parameter (CoAP POST) to server'
      - - Type = 0 (CON)
        - Code = 2 (POST)

  - step_id: 'TD_LWM2M_1.0_INT_104_step_06'
    type: check
    description:
        - 'The request sent by the client contains'
        - - Type=0 and Code=2
          - Uri-Path option= /rd

  - step_id: 'TD_LWM2M_1.0_INT_104_step_07'
    type: check
    description:
        - 'Server sends response containing'
        - - Code = 2.04 (Changed)

  - step_id: 'TD_LWM2M_1.0_INT_104_step_08'
    type: verify
    node: lwm2m_client
    description:
        - 'Client has received Success message from Server'
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='post')]

    def run(self):
        self.match('server', CoAP(type='con', code='post', opt=self.uri('/1/0/8')), 'fail')
        self.match('server', CoAP(pl=(b'')))
        
        self.next()

        self.match('client', CoAP(type='con', code=Any(65, 68)), 'fail')
    
        self.next()

        self.match('client', CoAP(type='con', code='post', opt=self.uri('/rd')), 'fail')
  
        self.next()

        self.match('server', CoAP(type='con', code=Any(65,68)), 'fail')



from ..common import *


class TD_COAP_OBS_07(CoAPTestCase):
    """
---
TD_COAP_OBS_07:
    obj: Server cleans the observers list on DELETE
    cfg: CoAP_CFG_BASIC
    ref: 3.2.4
    pre:
        - Client supports Observe option
        - Server supports Observe option
        - "Server offers an observable resource /obs which changes periodically (e.g. every
        5s) which produces confirmable notifications"

    seq:
    -   s: "Client is requested to send to the server a confirmable GET
            request with observe option for resource /obs"

    -   c:
        - The request sent by client contains:
        -   - Type = 0 (CON)
            - Code = 1 (GET)
            - Token value  = a value generated by the client
            - Observe option = empty

    -   c:
        - 'Server sends the response containing:'
        -   - Type = 2 (ACK)
            - Code = 2.05 (Content)
            - Content-format of the resource /obs
            - Token value = same as one found in the step 2
            - Observe option with a sequence number

    -   c:
        - 'Server sends a notification containing:'
        -   - Type = 0 (CON)
            - Code = 2.05 (Content)
            - Content-format = same as one found in the step 3
            - Token value = same as one found in the step 2
            - Observe option indicating increasing values


    -   c:
        - Client displays the received information

    -   c:
        - Client sends an ACK

    -   s: "Delete the /obs resource of the server (either locally or by
        having another CoAP client perform a DELETE request)"

    -   c:
        - 'Server sends a notification containing:'
        -   - Type = 0 (CON)
            - Code = 132 (4.04 NOT FOUND)
            - Token value = same as one found in the step 2
            - No Observe option any more

    -   v: Server does not send further responses

    -   v: Client does not display further received information

    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='con', code='get', opt=Opt(CoAPOptionObserve(0),CoAPOptionUriPath("obs"))),
            CoAP(type="con", code="delete", opt=Opt(CoAPOptionUriPath("obs"))),
        ]


    def run(self):
        request = CoAP(type="con", code="get",	opt=self.uri("/obs", CoAPOptionObserve(0)))
        response = CoAP(type="ack", code=2.05, opt=Opt(CoAPOptionObserve(), CoAPOptionContentFormat()))

        # Step 2
        self.match("client", request)

        token = self.coap["tok"]
        uri = self.coap.get_uri()

        self.next()

        # Step 3
        if not self.match("server", response):
            raise self.Stop()
        self.match("server", CoAP(tok = token), "fail")

        content_format = self.coap["opt"][CoAPOptionContentFormat]["val"]
        index          = self.coap["opt"][CoAPOptionObserve]["val"]

        self.next()

        # we need to observe at least one notification
        # (to ensure that the server is observing the resource for the client)

        verdict_if_none = "inconclusive"

        # Step 4
        while self.match("server", CoAP(type="con", code=2.05, opt=Opt (CoAPOptionObserve())),
            verdict_if_none):

            self.match("server", CoAP(tok = token), "fail")
            self.match("server", CoAP(opt = Opt (CoAPOptionContentFormat (content_format))), "fail")

            new_index = self.coap["opt"][CoAPOptionObserve]["val"]

            self.set_verdict(("pass" if new_index > index else "fail"),
            "value of observe option must be increasing")

            self.next()

            # Step 6
            self.match("client", CoAP (type="ack", code=0))

            self.next()

            # now we have successfully observed a observe response
            verdict_if_none = None

        # As we use UDP and we have two clients (one observing, another deleting),
        # We have no guarantee on which order will the packets arrive.
        # Hence we rely on the CMID to check the step 8.
        CMID_delete_request = None

        # Step 7
        if self.match("client", CoAP(type="con", code="delete"),
            None):

            self.set_verdict("pass" if uri == self.coap.get_uri() else "inconclusive",
                "deleted resource should be the observed resource (%s)" % uri)

            CMID_delete_request = self.coap["mid"]
            self.next()

        client_has_been_notified = False
        # Optional as the stimuli may be done internally by the server, as
        # stated by step 7.
        server_has_confirmed_deletion = False
        sent_notif_after_deletion = False

        while (client_has_been_notified is False and
            Not(CoAP(type="con", code=4.04, opt = NoOpt(CoAPOptionObserve())))
            and not sent_notif_after_deletion):
            if (self.coap["mid"] == CMID_delete_request):
                if self.match("server", CoAP(type="ack", code=2.02)):
                    server_has_confirmed_deletion = True
                self.next()
            # Step 8 - PASS case
            if self.match("server", CoAP(type="con", code=4.04, opt = NoOpt(CoAPOptionObserve())), None):
                self.match("server", CoAP(tok = token), "fail")
                client_has_been_notified = True
                self.next()
                self.match("client", CoAP(type="ack", code=0)) # Step 9

            if self.match("server", CoAP(type="con", code=2.05, opt=Opt (CoAPOptionObserve())),None):
                # Step 8 - FAIL case
                if server_has_confirmed_deletion:
                    sent_notif_after_deletion = True
                    self.set_verdict("fail", "Step 8-9: The server continued \
to send notification after it's confirmed the deletion of the resource")
                    self.next()
                else:
                    # Step 8 - INCONCLUSIVE case
                    if CMID_delete_request is not None: #i.e stimulus not done locally.
                        sent_notif_after_deletion = True
                        self.set_verdict("inconclusive", "The DELETE stimulis \
is present but there is no confirmation from the server (2.02 DELETED),\n nor 4.04\
 NOT FOUND sent to the observing client. Cannot determine if the stimulis was\
 actually executed.")
                        self.next()
                while self.match("server", CoAP(type="con", code=2.05, opt=Opt (CoAPOptionObserve())),
                None):
                    self.set_verdict("fail", "Step 8-9: The server continued\
 to send notification after it's confirmed the deletion of the resource")
                    self.next()



from ..common import *
from ...core.templates import Range

class TD_COAP_BLOCK_05 (CoAPTestCase):
    """
---
TD_COAP_BLOCK_05:
    obj: Handle POST with two-way blockwise transfer
    cfg: CoAP_CFG_BASIC
    ref: '[BLOCK] 2.2, 2.3, 2.5'

    pre:
        - Client supports Block1 and Block2 transfers
        - Server supports Block1 and Block2 transfers
        - "Server accepts large post requests on /large-post"

    seq:
    -   s: "Client is requested to send a large represenation to /large-post on Server"

    -   c:
    - 'Client sends a POST request containing Block1 option indicating:'
    -   - NUM = 0
        - M = 1 (more)
        - "SZX (➔DES_SZX) is the desired block size"
        - "Payload size is 2**(SZX+4) bytes"

    -   c:
        - 'Server sends 2.31 (Continue) response containing Block1 option indicating:'
        -   - NUM = 0
            - M = 1 (atomic)
            - "SZX (➔ACT_SZX) is less or equal to DES_SZX"

    -   c:
        - 'Client sends further POST requests containing Block1 option indicating:'
        -   - 'NUM = i where “i” is the block number of the current block.
            If the server decreased the SZX parameter in step 3,
            then the client needs to adapt the block size accordingly
            and resumes the transfer from block number 2**(DES_SZX - ACT_SZX)
            instead of block 1.'
            - M = 1 (more)
            - SZX is ACT_SZX
            - "Payload size is 2**(SZX+4) bytes"

    -   c:
        - 'Server sends 2.31 (Continue) response containing Block1 option indicating:'
        -   - NUM = i where “i” is the block number used at step 4
            - M = 1 (atomic)
            - SZX is ACT_SZX

    -   c:
        - 'Client sends POST request containing the last block and indicating:'
        -   - NUM = n where “n” is the last block number
            - M = 0 (final)
            - SZX is ACT_SZX
            - 'Payload size is less than or equal to 2**(SZX+4) bytes.'

    -   c:
        - 'Server sends 2.04 (Changed) response containing Block1 option indicating:'
        -   - NUM = n where “n” is the block number used at step 6
            - M = 0 (final)
            - SZX is ACT_SZX
        - 'and a Block2 option indicating:'
        -   - NUM = 0
            - M = 1 (more)
            - 'SZX (➔rDES_SZX) is the desired block size'
            - 'Payload size is 2**(SZX+4) bytes'

    -   c:
        - 'Client switches to blockwise retrieval of response
        and sends a POST request, with the same options except for Block1,
        without payload, with a Block2 option indicating:'
        -   - 'NUM is the next block number k = (2**(rDES_SZX – rACT_SZX))'
            - M = 0
            - "SZX (➔rACT_SZX) is less than or equal to rDES_SZX"

    -   c:
        - "Server sends 2.04 (Changed) response with a Block2 option indicating:"
        -   - NUM = k where “k” is the block number used at step 8
            - M = 1
            - SZX is rACT_SZX
            - 'Payload size is 2**(SZX+4) bytes'

    -   c:
        - 'Client sends a similar POST request for retrieving a further block
        indicating:'
        -   - NUM = i where “i” is the block number of the current block
            - M = 0
            - SZX is rACT_SZX

    -   c:
        - 'Server sends 2.04 (Changed) response with a Block2 option indicating:'
        -   - NUM = i where “i” is the block number used at step 10
            - M = 1
            - SZX is rACT_SZX
            - 'Payload size is 2**(SZX+4) bytes'

    -   c:
        - 'Client sends another POST request (which will retrieve the last block)
            indicating:'
        -   -NUM = n where “n” is the last block number
            - M = 0
            - SZX is rACT_SZX.

    -   c:
        - 'Server sends 2.04 (Changed) response with a Block2 option indicating:'
        -   - NUM = n where “n” is the block number used at step 12
            - M = 0
            - SZX is rACT_SZX
            - Payload size is less than or equal to 2**(SZX+4) bytes

    -   v: Client displays the response

    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='con', code='post', opt=Opt(CoAPOptionUriPath("large-post"))),
        ]

    def run (self):
        # Step 2
        self.match("client",CoAP(type="con",code="post",opt=self.uri("/large-post", CoAPOptionBlock1(num=0, m=1))))
        client_szx  = self.coap["opt"][CoAPOptionBlock1]["szx"]
        client_size = 2**(client_szx+4)
        self.match("client", CoAP (pl = Length(bytes, client_size)), "fail")

        self.next()
        # Step 3
        if self.match("server",CoAP(type="ack",code=2.31,opt=Opt(CoAPOptionBlock1(num=0,m=1)))):
            if not self.match("server", CoAP (opt = Opt (CoAPOptionBlock1 (szx=Range(int, 0, client_szx))))):
                raise self.Stop()
            server_szx  = self.coap["opt"][CoAPOptionBlock1]["szx"]
            server_size = 2**(server_szx+4)
            self.next()
            j=1
            while 1 :
                # Step 4
                self.match("client",CoAP(type="con",code="post",opt=Opt(CoAPOptionBlock1(num=j,szx=server_szx))),"fail")
                self.next()
                # Step 5
                self.match("server",CoAP(type="ack",opt=Opt(CoAPOptionBlock1(num=j))),"fail")
                # Step 6
                if not self.match("server",CoAP(code=2.31,opt=Opt(CoAPOptionBlock1(m=1))),None):
                    break
                self.next()
                j+=1
            # Step 7
            if self.match("server",CoAP(code=2.04,opt=Opt(CoAPOptionBlock1(m=0),CoAPOptionBlock2(num=0,m=1)))) :
                self.next()
                j=1
                while 1 :
                    # Step 8
                    self.match("client",CoAP(type="con",code="post",opt=Opt(CoAPOptionBlock2(num=j,m=0))),"fail")
                    self.next()
                    # Step 9
                    self.match("server",CoAP(type="ack",code=2.04, opt=Opt(CoAPOptionBlock2(num=j))),"fail")
                    # Step 10
                    if not self.match("server",CoAP(opt=Opt(CoAPOptionBlock2(m=1))),None):
                        break
                    self.next()
                    j+=1
        raise self.Stop()



from ..common import *

class TD_COAP_OBS_02(CoAPTestCase):
    """
---
TD_COAP_OBS_02:
    cfg: CoAP_CFG_BASIC
    obj: Handle resource observation with NON messages
    ref: '[OBSERVE] 1.2, 3, 4'

    pre:
        - Client supports Observe option
        - Server supports Observe option
        - "Server offers an observable resource /obs-non which changes
           periodically (e.g. every 5s) which produces non-confirmable
           notifications"

    seq:
        -   s: "Client is requested to send to the server a non-confirmable
                GET request with observe option for resource /obs-non"

        -   c:
            - 'The request sent by client contains:'
            -   - Type = 1 (NON)
                - Code = 1 (GET)
                - Token value  = a value generated by the client
                - Observe option = empty

        -   c:
        - 'Server sends a notification containing:'
        -    -Type = 1 (NON)
            - Code = 2.05 (Content)
            - Content-format = the same for all notifications
            - Token value = same as one found in the step 2
            - Observe option indicating increasing values

        -   v: Client displays the received information

    """
    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [
            CoAP(type='non', code='get', opt=Opt(CoAPOptionObserve(0), CoAPOptionUriPath("obs-non")))
        ]

    def run(self):
        # Step 2
        self.match("client", CoAP (code="get",
        opt=self.uri("/obs-non", CoAPOptionObserve (0))))

        token = self.coap["tok"]

        self.next()
        if self.match("server",CoAP(type="ack"),None):
            if not self.match("server",
                              CoAP(code=2.05,
                                   opt=Opt(CoAPOptionObserve(),
                                           CoAPOptionContentFormat()
                                           )
                                   )
                              ):
                raise self.Stop()

            content_format = self.coap["opt"][CoAPOptionContentFormat]["val"]
            index          = self.coap["opt"][CoAPOptionObserve]["val"]
            self.next()
        else:
            content_format = self.coap["opt"][CoAPOptionContentFormat]["val"]
            index          = self.coap["opt"][CoAPOptionObserve]["val"]
            self.next()

            # match at least 2 other notifications
            for i in range (2):
                # Step 3
                if not self.match("server", CoAP(type="non", code=2.05, opt=Opt(
                CoAPOptionObserve(),
                CoAPOptionContentFormat(),
                ))):
                    raise self.Stop()
                self.match("server", CoAP(tok = token), "fail")
                self.match("server", CoAP(opt = Opt (CoAPOptionContentFormat(content_format))), "fail")

                new_index = self.coap["opt"][CoAPOptionObserve]["val"]

                self.next()
                self.set_verdict (("pass" if new_index > index else "fail"),
                "value of observe option must be increasing")
                raise self.Stop()



#!/usr/bin/env python3

from ..common import *


class TD_COAP_CORE_19(CoAPTestCase):
    """
---
TD_COAP_CORE_19:
    cfg: CoAP_CFG_BASIC
    obj: Perform POST transaction with responses containing several
         Location-Query options (CON mode)
    pre: Server accepts creation of new resource on uri /location-query,
        the location of the created resource contains two query parameters
        ?first=1&second=2
    ref: '[COAP] 5.8.1, 5.10.8, 5.9.1.1'
    seq:
    -   s: "Client is requested to send a confirmable POST request to server\
            \u2019s resource"
    -   c:
        - 'The request sent by the client contains:'
        -   - Type = 0 (CON)
            - Code = 2 (POST)
            - "Client-generated Message ID (\u2794 CMID)"
            - "Client-generated Token (\u2794 CTOK)"
            - Content-format option
            - Empty or non-empty Payload
            - Uri-Path option "location-query"
    -   c:
        - 'Server sends response containing:'
        -   - Code = 2.01 (Created)
            - Message ID = CMID, Token = CTOK
            - Content-format option if payload non-empty
            - Zero or more Location-path options
            - Empty or non-empty Payload
        - 'and two options of type Location-Query, with the values (none
            of which contains a "?" or "&"):'
        -   - first=1
            - second=2
    -   v: Client displays the response
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]

        .. note::
            Check the number/value of the uri query options or not?
        """
        return [
            CoAP(type='con', code='post', opt=Opt(CoAPOptionUriPath("location-query")))
        ]

    def run(self):
        self.match("client", CoAP(type="con",
                                  code="post",
                                  opt=self.uri(
                                      "/location-query",
                                      CoAPOptionContentFormat(),
                                  )))
        CMID = self.coap["mid"]
        CTOK = self.coap["tok"]

        self.next_skip_ack()
        # TODO: generate a fail if we have a '&' or '?' or '.' or '..' in the CoAPOptionLocationQuery ? see 5.10.7
        self.match("server", CoAP(type=Any(CoAPType("con"), "ack"),
                                  code=2.01,
                                  mid=CMID,
                                  tok=CTOK,
                                  opt=Opt(
                                      CoAPOptionLocationQuery("first=1"),
                                      CoAPOptionLocationQuery("second=2"),
                                  )))

        if self.match('server', CoAP(pl=Not(b'')), None):
            self.match("server", CoAP(opt=Opt(CoAPOptionContentFormat()), ), "fail")

        self.next_skip_ack(optional=True)



from ..common import *


class TD_m2m_nh_13 (CoAPTestCase):
    """
---
TD_M2M_NH_13:
    cfg: M2M_CFG_01
    obj: AE deletes a specific container resource via a container DELETE Request
    pre: AE has created an Application Entity resource <AE> on Registrar CSE and
         AE has created a container resource <container> on Registrar CSE.
    ref: 'TS-0001, clause 10.2.4.4 and TS-0004, clause 7.3.5.2.4'
    seq:
    -   s:
        - 'AE is requested to send a <container> Delete Request:'
        -   - Type = 0 (CON)
            - Code = 4 (DELETE)
    -   c:
        - 'The request sent by the client contains:'
        -   - Type=0 and Code=4
            - "Client-generated Message ID (\u2794 CMID)"
            - "Client-generated Token (\u2794 CTOK)"
            - oneM2M-FR=AE-ID
            - oneM2M-RQI=token-string (CRQI)
            - Empty payload
    -   v: Check if possible that the <container> resource is deleted in registrar CSE
    -   c:
        - 'Registrar CSE sends response containing:'
        -   - Code = 2.02 (OK)
            - Message ID = CMID, Token = CTOK
            - Empty Payload
            - oneM2M-RSC=2002
            - oneM2M-RQI=CRQI
    -   v: Check if possible that the <AE> resource has been removed from registrar CSE
    -   v: AE indicates a successuful operation
    """

    @classmethod
    @typecheck
    def get_stimulis(cls) -> list_of(Value):
        """
        Get the stimulis of this test case. This has to be be implemented into
        each test cases class.

        :return: The stimulis of this TC
        :rtype: [Value]
        """
        return [CoAP(type='con', code='delete')]

    def run(self):
        
        self.match('client', CoAP(type='con', code='delete', pl=(b'')), 'fail')
        self.match('client', CoAP(opt=Opt(CoAPOptionOneM2MFrom())), 'fail')
        if self.match('client', CoAP(opt=Opt(CoAPOptionOneM2MRequestIdentifier())), 'fail'): 

            CMID = self.coap['mid']
            CTOK = self.coap['tok']
            OPTS = self.coap['opt']
            RI = OPTS[CoAPOptionOneM2MRequestIdentifier]
            RIVAL = RI[2]

            self.next()

            self.match('server', CoAP(code=2.02, mid=CMID, tok=CTOK, pl=(b'')), 'fail')
            self.match('server', CoAP(opt=Opt(CoAPOptionOneM2MResponseStatusCode('2002'))), 'fail')
            self.match('server', CoAP(opt=Opt(CoAPOptionOneM2MRequestIdentifier(RIVAL))), 'fail')
